searchData={"items":[{"type":"module","title":"Reactor.Req","doc":"An extension which provides direct support for working with `req` requests\nwithin Reactor.","ref":"Reactor.Req.html"},{"type":"module","title":"Reactor.Req.Builder","doc":"Common builder for all DSL entities","ref":"Reactor.Req.Builder.html"},{"type":"module","title":"Reactor.Req.Dsl.Delete","doc":"A `req_delete` DSL entity for the `Reactor.Req` DSL extension.","ref":"Reactor.Req.Dsl.Delete.html"},{"type":"type","title":"Reactor.Req.Dsl.Delete.t/0","doc":"","ref":"Reactor.Req.Dsl.Delete.html#t:t/0"},{"type":"module","title":"Reactor.Req.Dsl.Get","doc":"A `req_get` DSL entity for the `Reactor.Req` DSL extension.","ref":"Reactor.Req.Dsl.Get.html"},{"type":"type","title":"Reactor.Req.Dsl.Get.t/0","doc":"","ref":"Reactor.Req.Dsl.Get.html#t:t/0"},{"type":"module","title":"Reactor.Req.Dsl.Head","doc":"A `req_head` DSL entity for the `Reactor.Req` DSL extension.","ref":"Reactor.Req.Dsl.Head.html"},{"type":"type","title":"Reactor.Req.Dsl.Head.t/0","doc":"","ref":"Reactor.Req.Dsl.Head.html#t:t/0"},{"type":"module","title":"Reactor.Req.Dsl.Merge","doc":"The `req_merge` DSL entity for the `Reactor.Req` DSL extension.","ref":"Reactor.Req.Dsl.Merge.html"},{"type":"type","title":"Reactor.Req.Dsl.Merge.t/0","doc":"","ref":"Reactor.Req.Dsl.Merge.html#t:t/0"},{"type":"module","title":"Reactor.Req.Dsl.New","doc":"The `req_new` DSL entity for the `Reactor.Req` DSL extension.","ref":"Reactor.Req.Dsl.New.html"},{"type":"type","title":"Reactor.Req.Dsl.New.t/0","doc":"","ref":"Reactor.Req.Dsl.New.html#t:t/0"},{"type":"module","title":"Reactor.Req.Dsl.Options","doc":"All the known options of Req (as of this writing).","ref":"Reactor.Req.Dsl.Options.html"},{"type":"type","title":"Reactor.Req.Dsl.Options.entity/0","doc":"","ref":"Reactor.Req.Dsl.Options.html#t:entity/0"},{"type":"function","title":"Reactor.Req.Dsl.Options.merge/1","doc":"Merge options","ref":"Reactor.Req.Dsl.Options.html#merge/1"},{"type":"function","title":"Reactor.Req.Dsl.Options.struct_attrs/0","doc":"Struct attributes","ref":"Reactor.Req.Dsl.Options.html#struct_attrs/0"},{"type":"module","title":"Reactor.Req.Dsl.Patch","doc":"A `req_patch` DSL entity for the `Reactor.Req` DSL extension.","ref":"Reactor.Req.Dsl.Patch.html"},{"type":"type","title":"Reactor.Req.Dsl.Patch.t/0","doc":"","ref":"Reactor.Req.Dsl.Patch.html#t:t/0"},{"type":"module","title":"Reactor.Req.Dsl.Post","doc":"A `req_post` DSL entity for the `Reactor.Req` DSL extension.","ref":"Reactor.Req.Dsl.Post.html"},{"type":"type","title":"Reactor.Req.Dsl.Post.t/0","doc":"","ref":"Reactor.Req.Dsl.Post.html#t:t/0"},{"type":"module","title":"Reactor.Req.Dsl.Put","doc":"A `req_put` DSL entity for the `Reactor.Req` DSL extension.","ref":"Reactor.Req.Dsl.Put.html"},{"type":"type","title":"Reactor.Req.Dsl.Put.t/0","doc":"","ref":"Reactor.Req.Dsl.Put.html#t:t/0"},{"type":"module","title":"Reactor.Req.Dsl.Request","doc":"A `req_run` DSL entity for the `Reactor.Req` DSL extension.","ref":"Reactor.Req.Dsl.Request.html"},{"type":"type","title":"Reactor.Req.Dsl.Request.t/0","doc":"","ref":"Reactor.Req.Dsl.Request.html#t:t/0"},{"type":"module","title":"Reactor.Req.Dsl.Run","doc":"A `req_run` DSL entity for the `Reactor.Req` DSL extension.","ref":"Reactor.Req.Dsl.Run.html"},{"type":"type","title":"Reactor.Req.Dsl.Run.t/0","doc":"","ref":"Reactor.Req.Dsl.Run.html#t:t/0"},{"type":"module","title":"Reactor.Req.Step","doc":"A step which delegates to `req`.","ref":"Reactor.Req.Step.html"},{"type":"extras","title":"README","doc":"# Reactor.Req\n\n[![Build Status](https://github.com/ash-project/reactor_req/actions/workflows/elixir.yml/badge.svg)](https://github.com/ash-project/reactor/actions)\n[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)\n[![Hex version badge](https://img.shields.io/hexpm/v/reactor_req.svg)](https://hex.pm/packages/reactor_req)\n\nA [Reactor](https://github.com/ash-project/reactor) extension that provides steps for working with HTTP requests via [Req](https://github.com/wojtekmach/req).","ref":"readme.html"},{"type":"extras","title":"Example - README","doc":"The following example uses Reactor to retrieve the repository description from the GitHub API:\n\n```elixir\ndefmodule GetGitHubRepoDescription do\n  use Reactor, extensions: [Reactor.Req]\n\n  input :owner\n  input :repo\n\n  step :repo_url do\n    argument :owner, input(:owner)\n    argument :repo, input(:repo)\n\n    run fn args ->\n      URI.new(\"https://api.github.com/repos/#{args.owner}/#{args.repo}\")\n    end\n  end\n\n  req_get :get_repo do\n    url result(:repo_url)\n    headers value([accept: \"application/vnd.github.v3+json\"])\n    http_errors value(:raise)\n  end\n\n  step :get_description do\n    argument :description, result(:get_repo, [:body, \"description\"])\n    run fn args -> {:ok, args.description} end\n  end\nend\n\nReactor.run!(GetGitHubRepoDescription, %{\n  owner: \"ash-project\",\n  repo: \"reactor_req\"\n})\n\n# => \"A Reactor DSL extension for making HTTP requests with Req.\"\n```","ref":"readme.html#example"},{"type":"extras","title":"Installation - README","doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `reactor_req` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:reactor_req, \"~> 0.1.4\"}\n  ]\nend\n```\n\nDocumentation for the latest release is available on [HexDocs](https://hexdocs.pm/reactor_req).","ref":"readme.html#installation"},{"type":"extras","title":"Licence - README","doc":"This software is licensed under the terms of the MIT License.\nSee the `LICENSE` file included with this package for the terms.","ref":"readme.html#licence"},{"type":"extras","title":"Reactor.Req","doc":"<!--\nThis file was generated by Spark. Do not edit it by hand.\n-->\n# Reactor.Req\n\nAn extension which provides direct support for working with `req` requests\nwithin Reactor.","ref":"dsl-reactor-req.html"},{"type":"extras","title":"reactor.req_delete - Reactor.Req","doc":"```elixir\nreq_delete name\n```\n\n\nPerforms a request using `Req.delete/2`.\n\nNote that Reactor doesn't validate any options - it simply passes them to the underlying `Req` function and assumes it will validate them.","ref":"dsl-reactor-req.html#reactor-req_delete"},{"type":"extras","title":"Nested DSLs - Reactor.Req","doc":"* [wait_for](#reactor-req_delete-wait_for)\n * [guard](#reactor-req_delete-guard)\n * [where](#reactor-req_delete-where)","ref":"dsl-reactor-req.html#nested-dsls"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`name`](#reactor-req_delete-name){: #reactor-req_delete-name .spark-required} | `atom` |  | A unique name for the step. Used when choosing the return value of the Reactor and for arguments into other steps |","ref":"dsl-reactor-req.html#arguments"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`adapter`](#reactor-req_delete-adapter){: #reactor-req_delete-adapter } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Adapter to use to make the actual HTTP request |\n| [`auth`](#reactor-req_delete-auth){: #reactor-req_delete-auth } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Sets request authentication |\n| [`aws_sigv4`](#reactor-req_delete-aws_sigv4){: #reactor-req_delete-aws_sigv4 } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, the AWS options to sign request |\n| [`base_url`](#reactor-req_delete-base_url){: #reactor-req_delete-base_url } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, the request URL is prepended with this base URL |\n| [`body`](#reactor-req_delete-body){: #reactor-req_delete-body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request body |\n| [`cache_dir`](#reactor-req_delete-cache_dir){: #reactor-req_delete-cache_dir } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The directory to store the cache |\n| [`cache`](#reactor-req_delete-cache){: #reactor-req_delete-cache } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If `true`, performs HTTP caching |\n| [`compress_body`](#reactor-req_delete-compress_body){: #reactor-req_delete-compress_body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, compresses the request body using gzip |\n| [`connect_options`](#reactor-req_delete-connect_options){: #reactor-req_delete-connect_options } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Dynamically starts (or re-uses already started) Finch pool with the given connection options |\n| [`decode_body`](#reactor-req_delete-decode_body){: #reactor-req_delete-decode_body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `false`, disables automatic response body decoding |\n| [`decode_json`](#reactor-req_delete-decode_json){: #reactor-req_delete-decode_json } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Options to pass to `Jason.decode!/2` |\n| [`finch_private`](#reactor-req_delete-finch_private){: #reactor-req_delete-finch_private } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A map or keyword list of private metadata to add to the Finch request |\n| [`finch_request`](#reactor-req_delete-finch_request){: #reactor-req_delete-finch_request } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A function that executes the Finch request, defaults to using `Finch.request/3` |\n| [`finch`](#reactor-req_delete-finch){: #reactor-req_delete-finch } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The Finch pool to use. Defaults to pool automatically started by `Req` |\n| [`form_multipart`](#reactor-req_delete-form_multipart){: #reactor-req_delete-form_multipart } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as `multipart/form-data` |\n| [`form`](#reactor-req_delete-form){: #reactor-req_delete-form } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as `application/x-www-form-urlencoded` |\n| [`headers`](#reactor-req_delete-headers){: #reactor-req_delete-headers } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request headers as a `{key, value}` enumerable (e.g. map, keyword list) |\n| [`http_errors`](#reactor-req_delete-http_errors){: #reactor-req_delete-http_errors } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | How to manage 4xx and 5xx responses |\n| [`inet6`](#reactor-req_delete-inet6){: #reactor-req_delete-inet6 } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, uses IPv6 |\n| [`into`](#reactor-req_delete-into){: #reactor-req_delete-into } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Where to send the response body |\n| [`json`](#reactor-req_delete-json){: #reactor-req_delete-json } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as JSON |\n| [`max_redirects`](#reactor-req_delete-max_redirects){: #reactor-req_delete-max_redirects } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The maximum number of redirects, defaults to `10` |\n| [`max_retries`](#reactor-req_delete-max_retries){: #reactor-req_delete-max_retries } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Maximum number of retry attempts, defaults to `3` (for a total of `4` requests to the server, including the initial one) |\n| [`method`](#reactor-req_delete-method){: #reactor-req_delete-method } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request method, defaults to `:get` |\n| [`params`](#reactor-req_delete-params){: #reactor-req_delete-params } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, appends parameters to the request query string (via `put_params` step) |\n| [`path_params_style`](#reactor-req_delete-path_params_style){: #reactor-req_delete-path_params_style } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | How path params are expressed (via `put_path_params` step) |\n| [`path_params`](#reactor-req_delete-path_params){: #reactor-req_delete-path_params } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, uses a templated request path (via `put_path_params` step) |\n| [`plug`](#reactor-req_delete-plug){: #reactor-req_delete-plug } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, calls the given plug instead of making an HTTP request over the network (via `run_plug` step) |\n| [`pool_timeout`](#reactor-req_delete-pool_timeout){: #reactor-req_delete-pool_timeout } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Pool checkout timeout in milliseconds, defaults to `5000` |\n| [`raw`](#reactor-req_delete-raw){: #reactor-req_delete-raw } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, disables automatic body decompression (`decompress_body` step) and decoding (`decode_body` step) |\n| [`receive_timeout`](#reactor-req_delete-receive_timeout){: #reactor-req_delete-receive_timeout } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Socket receive timeout in milliseconds, defaults to `15_000` |\n| [`redirect_trusted`](#reactor-req_delete-redirect_trusted){: #reactor-req_delete-redirect_trusted } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | By default, authorization credentials are only sent on redirects with the same host, scheme and port. If `:redirect_trusted` is set to `true`, credentials will be sent to any host |\n| [`redirect`](#reactor-req_delete-redirect){: #reactor-req_delete-redirect } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `false`, disables automatic response redirects |\n| [`request`](#reactor-req_delete-request){: #reactor-req_delete-request } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A previously built request |\n| [`retry_delay`](#reactor-req_delete-retry_delay){: #reactor-req_delete-retry_delay } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If not set, which is the default, the retry delay is determined by the value of retry-delay header on HTTP 429/503 responses. If the header is not set, the default delay follows a simple exponential backoff: 1s, 2s, 4s, 8s, ... |\n| [`retry_log_level`](#reactor-req_delete-retry_log_level){: #reactor-req_delete-retry_log_level } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The log level to emit retry logs at. Can also be set to `false` to disable logging these messages. Defaults to `:warning` |\n| [`retry`](#reactor-req_delete-retry){: #reactor-req_delete-retry } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | One of `:safe_transient` (default), `:transient`, `fun` or `false` |\n| [`unix_socket`](#reactor-req_delete-unix_socket){: #reactor-req_delete-unix_socket } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, connect through the given UNIX domain socket |\n| [`url`](#reactor-req_delete-url){: #reactor-req_delete-url } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request URL |","ref":"dsl-reactor-req.html#options"},{"type":"extras","title":"reactor.req_delete.wait_for - Reactor.Req","doc":"```elixir\nwait_for names\n```\n\n\nWait for the named step to complete before allowing this one to start.\n\nDesugars to `argument :_, result(step_to_wait_for)`","ref":"dsl-reactor-req.html#reactor-req_delete-wait_for"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nwait_for :create_user\n```","ref":"dsl-reactor-req.html#examples"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`names`](#reactor-req_delete-wait_for-names){: #reactor-req_delete-wait_for-names .spark-required} | `atom \\| list(atom)` |  | The name of the step to wait for. |","ref":"dsl-reactor-req.html#arguments-1"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_delete-wait_for-description){: #reactor-req_delete-wait_for-description } | `String.t` |  | An optional description. |","ref":"dsl-reactor-req.html#options-1"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.WaitFor`","ref":"dsl-reactor-req.html#introspection"},{"type":"extras","title":"reactor.req_delete.guard - Reactor.Req","doc":"```elixir\nguard fun\n```\n\n\nProvides a flexible method for conditionally executing a step, or replacing it's result.\n\nExpects a two arity function which takes the step's arguments and context and returns one of the following:\n\n- `:cont` - the guard has passed.\n- `{:halt, result}` - the guard has failed - instead of executing the step use the provided result.","ref":"dsl-reactor-req.html#reactor-req_delete-guard"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nstep :read_file_via_cache do\n  argument :path, input(:path)\n  run &File.read(&1.path)\n  guard fn %{path: path}, %{cache: cache} ->\n    case Cache.get(cache, path) do\n      {:ok, content} -> {:halt, {:ok, content}}\n      _ -> :cont\n    end\n  end\nend\n\n```","ref":"dsl-reactor-req.html#examples-1"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`fun`](#reactor-req_delete-guard-fun){: #reactor-req_delete-guard-fun .spark-required} | `(any, any -> any) \\| mfa` |  | The guard function. |","ref":"dsl-reactor-req.html#arguments-2"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_delete-guard-description){: #reactor-req_delete-guard-description } | `String.t` |  | An optional description of the guard. |","ref":"dsl-reactor-req.html#options-2"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.Guard`","ref":"dsl-reactor-req.html#introspection-1"},{"type":"extras","title":"reactor.req_delete.where - Reactor.Req","doc":"```elixir\nwhere predicate\n```\n\n\nOnly execute the surrounding step if the predicate function returns true.\n\nThis is a simple version of `guard` which provides more flexibility at the cost of complexity.","ref":"dsl-reactor-req.html#reactor-req_delete-where"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nstep :read_file do\n  argument :path, input(:path)\n  run &File.read(&1.path)\n  where &File.exists?(&1.path)\nend\n\n```","ref":"dsl-reactor-req.html#examples-2"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`predicate`](#reactor-req_delete-where-predicate){: #reactor-req_delete-where-predicate .spark-required} | `(any -> any) \\| mfa \\| (any, any -> any) \\| mfa` |  | Provide a function which takes the step arguments and optionally the context and returns a boolean value. |","ref":"dsl-reactor-req.html#arguments-3"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_delete-where-description){: #reactor-req_delete-where-description } | `String.t` |  | An optional description of the guard. |","ref":"dsl-reactor-req.html#options-3"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.Where`","ref":"dsl-reactor-req.html#introspection-2"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Req.Dsl.Delete`","ref":"dsl-reactor-req.html#introspection-3"},{"type":"extras","title":"reactor.req_get - Reactor.Req","doc":"```elixir\nreq_get name\n```\n\n\nPerforms a request using `Req.get/2`.\n\nNote that Reactor doesn't validate any options - it simply passes them to the underlying `Req` function and assumes it will validate them.","ref":"dsl-reactor-req.html#reactor-req_get"},{"type":"extras","title":"Nested DSLs - Reactor.Req","doc":"* [wait_for](#reactor-req_get-wait_for)\n * [guard](#reactor-req_get-guard)\n * [where](#reactor-req_get-where)","ref":"dsl-reactor-req.html#nested-dsls-1"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`name`](#reactor-req_get-name){: #reactor-req_get-name .spark-required} | `atom` |  | A unique name for the step. Used when choosing the return value of the Reactor and for arguments into other steps |","ref":"dsl-reactor-req.html#arguments-4"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`adapter`](#reactor-req_get-adapter){: #reactor-req_get-adapter } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Adapter to use to make the actual HTTP request |\n| [`auth`](#reactor-req_get-auth){: #reactor-req_get-auth } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Sets request authentication |\n| [`aws_sigv4`](#reactor-req_get-aws_sigv4){: #reactor-req_get-aws_sigv4 } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, the AWS options to sign request |\n| [`base_url`](#reactor-req_get-base_url){: #reactor-req_get-base_url } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, the request URL is prepended with this base URL |\n| [`body`](#reactor-req_get-body){: #reactor-req_get-body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request body |\n| [`cache_dir`](#reactor-req_get-cache_dir){: #reactor-req_get-cache_dir } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The directory to store the cache |\n| [`cache`](#reactor-req_get-cache){: #reactor-req_get-cache } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If `true`, performs HTTP caching |\n| [`compress_body`](#reactor-req_get-compress_body){: #reactor-req_get-compress_body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, compresses the request body using gzip |\n| [`connect_options`](#reactor-req_get-connect_options){: #reactor-req_get-connect_options } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Dynamically starts (or re-uses already started) Finch pool with the given connection options |\n| [`decode_body`](#reactor-req_get-decode_body){: #reactor-req_get-decode_body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `false`, disables automatic response body decoding |\n| [`decode_json`](#reactor-req_get-decode_json){: #reactor-req_get-decode_json } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Options to pass to `Jason.decode!/2` |\n| [`finch_private`](#reactor-req_get-finch_private){: #reactor-req_get-finch_private } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A map or keyword list of private metadata to add to the Finch request |\n| [`finch_request`](#reactor-req_get-finch_request){: #reactor-req_get-finch_request } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A function that executes the Finch request, defaults to using `Finch.request/3` |\n| [`finch`](#reactor-req_get-finch){: #reactor-req_get-finch } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The Finch pool to use. Defaults to pool automatically started by `Req` |\n| [`form_multipart`](#reactor-req_get-form_multipart){: #reactor-req_get-form_multipart } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as `multipart/form-data` |\n| [`form`](#reactor-req_get-form){: #reactor-req_get-form } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as `application/x-www-form-urlencoded` |\n| [`headers`](#reactor-req_get-headers){: #reactor-req_get-headers } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request headers as a `{key, value}` enumerable (e.g. map, keyword list) |\n| [`http_errors`](#reactor-req_get-http_errors){: #reactor-req_get-http_errors } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | How to manage 4xx and 5xx responses |\n| [`inet6`](#reactor-req_get-inet6){: #reactor-req_get-inet6 } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, uses IPv6 |\n| [`into`](#reactor-req_get-into){: #reactor-req_get-into } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Where to send the response body |\n| [`json`](#reactor-req_get-json){: #reactor-req_get-json } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as JSON |\n| [`max_redirects`](#reactor-req_get-max_redirects){: #reactor-req_get-max_redirects } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The maximum number of redirects, defaults to `10` |\n| [`max_retries`](#reactor-req_get-max_retries){: #reactor-req_get-max_retries } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Maximum number of retry attempts, defaults to `3` (for a total of `4` requests to the server, including the initial one) |\n| [`method`](#reactor-req_get-method){: #reactor-req_get-method } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request method, defaults to `:get` |\n| [`params`](#reactor-req_get-params){: #reactor-req_get-params } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, appends parameters to the request query string (via `put_params` step) |\n| [`path_params_style`](#reactor-req_get-path_params_style){: #reactor-req_get-path_params_style } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | How path params are expressed (via `put_path_params` step) |\n| [`path_params`](#reactor-req_get-path_params){: #reactor-req_get-path_params } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, uses a templated request path (via `put_path_params` step) |\n| [`plug`](#reactor-req_get-plug){: #reactor-req_get-plug } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, calls the given plug instead of making an HTTP request over the network (via `run_plug` step) |\n| [`pool_timeout`](#reactor-req_get-pool_timeout){: #reactor-req_get-pool_timeout } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Pool checkout timeout in milliseconds, defaults to `5000` |\n| [`raw`](#reactor-req_get-raw){: #reactor-req_get-raw } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, disables automatic body decompression (`decompress_body` step) and decoding (`decode_body` step) |\n| [`receive_timeout`](#reactor-req_get-receive_timeout){: #reactor-req_get-receive_timeout } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Socket receive timeout in milliseconds, defaults to `15_000` |\n| [`redirect_trusted`](#reactor-req_get-redirect_trusted){: #reactor-req_get-redirect_trusted } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | By default, authorization credentials are only sent on redirects with the same host, scheme and port. If `:redirect_trusted` is set to `true`, credentials will be sent to any host |\n| [`redirect`](#reactor-req_get-redirect){: #reactor-req_get-redirect } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `false`, disables automatic response redirects |\n| [`request`](#reactor-req_get-request){: #reactor-req_get-request } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A previously built request |\n| [`retry_delay`](#reactor-req_get-retry_delay){: #reactor-req_get-retry_delay } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If not set, which is the default, the retry delay is determined by the value of retry-delay header on HTTP 429/503 responses. If the header is not set, the default delay follows a simple exponential backoff: 1s, 2s, 4s, 8s, ... |\n| [`retry_log_level`](#reactor-req_get-retry_log_level){: #reactor-req_get-retry_log_level } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The log level to emit retry logs at. Can also be set to `false` to disable logging these messages. Defaults to `:warning` |\n| [`retry`](#reactor-req_get-retry){: #reactor-req_get-retry } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | One of `:safe_transient` (default), `:transient`, `fun` or `false` |\n| [`unix_socket`](#reactor-req_get-unix_socket){: #reactor-req_get-unix_socket } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, connect through the given UNIX domain socket |\n| [`url`](#reactor-req_get-url){: #reactor-req_get-url } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request URL |","ref":"dsl-reactor-req.html#options-4"},{"type":"extras","title":"reactor.req_get.wait_for - Reactor.Req","doc":"```elixir\nwait_for names\n```\n\n\nWait for the named step to complete before allowing this one to start.\n\nDesugars to `argument :_, result(step_to_wait_for)`","ref":"dsl-reactor-req.html#reactor-req_get-wait_for"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nwait_for :create_user\n```","ref":"dsl-reactor-req.html#examples-3"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`names`](#reactor-req_get-wait_for-names){: #reactor-req_get-wait_for-names .spark-required} | `atom \\| list(atom)` |  | The name of the step to wait for. |","ref":"dsl-reactor-req.html#arguments-5"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_get-wait_for-description){: #reactor-req_get-wait_for-description } | `String.t` |  | An optional description. |","ref":"dsl-reactor-req.html#options-5"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.WaitFor`","ref":"dsl-reactor-req.html#introspection-4"},{"type":"extras","title":"reactor.req_get.guard - Reactor.Req","doc":"```elixir\nguard fun\n```\n\n\nProvides a flexible method for conditionally executing a step, or replacing it's result.\n\nExpects a two arity function which takes the step's arguments and context and returns one of the following:\n\n- `:cont` - the guard has passed.\n- `{:halt, result}` - the guard has failed - instead of executing the step use the provided result.","ref":"dsl-reactor-req.html#reactor-req_get-guard"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nstep :read_file_via_cache do\n  argument :path, input(:path)\n  run &File.read(&1.path)\n  guard fn %{path: path}, %{cache: cache} ->\n    case Cache.get(cache, path) do\n      {:ok, content} -> {:halt, {:ok, content}}\n      _ -> :cont\n    end\n  end\nend\n\n```","ref":"dsl-reactor-req.html#examples-4"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`fun`](#reactor-req_get-guard-fun){: #reactor-req_get-guard-fun .spark-required} | `(any, any -> any) \\| mfa` |  | The guard function. |","ref":"dsl-reactor-req.html#arguments-6"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_get-guard-description){: #reactor-req_get-guard-description } | `String.t` |  | An optional description of the guard. |","ref":"dsl-reactor-req.html#options-6"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.Guard`","ref":"dsl-reactor-req.html#introspection-5"},{"type":"extras","title":"reactor.req_get.where - Reactor.Req","doc":"```elixir\nwhere predicate\n```\n\n\nOnly execute the surrounding step if the predicate function returns true.\n\nThis is a simple version of `guard` which provides more flexibility at the cost of complexity.","ref":"dsl-reactor-req.html#reactor-req_get-where"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nstep :read_file do\n  argument :path, input(:path)\n  run &File.read(&1.path)\n  where &File.exists?(&1.path)\nend\n\n```","ref":"dsl-reactor-req.html#examples-5"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`predicate`](#reactor-req_get-where-predicate){: #reactor-req_get-where-predicate .spark-required} | `(any -> any) \\| mfa \\| (any, any -> any) \\| mfa` |  | Provide a function which takes the step arguments and optionally the context and returns a boolean value. |","ref":"dsl-reactor-req.html#arguments-7"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_get-where-description){: #reactor-req_get-where-description } | `String.t` |  | An optional description of the guard. |","ref":"dsl-reactor-req.html#options-7"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.Where`","ref":"dsl-reactor-req.html#introspection-6"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Req.Dsl.Get`","ref":"dsl-reactor-req.html#introspection-7"},{"type":"extras","title":"reactor.req_head - Reactor.Req","doc":"```elixir\nreq_head name\n```\n\n\nPerforms a request using `Req.head/2`.\n\nNote that Reactor doesn't validate any options - it simply passes them to the underlying `Req` function and assumes it will validate them.","ref":"dsl-reactor-req.html#reactor-req_head"},{"type":"extras","title":"Nested DSLs - Reactor.Req","doc":"* [wait_for](#reactor-req_head-wait_for)\n * [guard](#reactor-req_head-guard)\n * [where](#reactor-req_head-where)","ref":"dsl-reactor-req.html#nested-dsls-2"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`name`](#reactor-req_head-name){: #reactor-req_head-name .spark-required} | `atom` |  | A unique name for the step. Used when choosing the return value of the Reactor and for arguments into other steps |","ref":"dsl-reactor-req.html#arguments-8"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`adapter`](#reactor-req_head-adapter){: #reactor-req_head-adapter } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Adapter to use to make the actual HTTP request |\n| [`auth`](#reactor-req_head-auth){: #reactor-req_head-auth } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Sets request authentication |\n| [`aws_sigv4`](#reactor-req_head-aws_sigv4){: #reactor-req_head-aws_sigv4 } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, the AWS options to sign request |\n| [`base_url`](#reactor-req_head-base_url){: #reactor-req_head-base_url } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, the request URL is prepended with this base URL |\n| [`body`](#reactor-req_head-body){: #reactor-req_head-body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request body |\n| [`cache_dir`](#reactor-req_head-cache_dir){: #reactor-req_head-cache_dir } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The directory to store the cache |\n| [`cache`](#reactor-req_head-cache){: #reactor-req_head-cache } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If `true`, performs HTTP caching |\n| [`compress_body`](#reactor-req_head-compress_body){: #reactor-req_head-compress_body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, compresses the request body using gzip |\n| [`connect_options`](#reactor-req_head-connect_options){: #reactor-req_head-connect_options } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Dynamically starts (or re-uses already started) Finch pool with the given connection options |\n| [`decode_body`](#reactor-req_head-decode_body){: #reactor-req_head-decode_body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `false`, disables automatic response body decoding |\n| [`decode_json`](#reactor-req_head-decode_json){: #reactor-req_head-decode_json } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Options to pass to `Jason.decode!/2` |\n| [`finch_private`](#reactor-req_head-finch_private){: #reactor-req_head-finch_private } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A map or keyword list of private metadata to add to the Finch request |\n| [`finch_request`](#reactor-req_head-finch_request){: #reactor-req_head-finch_request } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A function that executes the Finch request, defaults to using `Finch.request/3` |\n| [`finch`](#reactor-req_head-finch){: #reactor-req_head-finch } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The Finch pool to use. Defaults to pool automatically started by `Req` |\n| [`form_multipart`](#reactor-req_head-form_multipart){: #reactor-req_head-form_multipart } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as `multipart/form-data` |\n| [`form`](#reactor-req_head-form){: #reactor-req_head-form } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as `application/x-www-form-urlencoded` |\n| [`headers`](#reactor-req_head-headers){: #reactor-req_head-headers } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request headers as a `{key, value}` enumerable (e.g. map, keyword list) |\n| [`http_errors`](#reactor-req_head-http_errors){: #reactor-req_head-http_errors } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | How to manage 4xx and 5xx responses |\n| [`inet6`](#reactor-req_head-inet6){: #reactor-req_head-inet6 } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, uses IPv6 |\n| [`into`](#reactor-req_head-into){: #reactor-req_head-into } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Where to send the response body |\n| [`json`](#reactor-req_head-json){: #reactor-req_head-json } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as JSON |\n| [`max_redirects`](#reactor-req_head-max_redirects){: #reactor-req_head-max_redirects } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The maximum number of redirects, defaults to `10` |\n| [`max_retries`](#reactor-req_head-max_retries){: #reactor-req_head-max_retries } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Maximum number of retry attempts, defaults to `3` (for a total of `4` requests to the server, including the initial one) |\n| [`method`](#reactor-req_head-method){: #reactor-req_head-method } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request method, defaults to `:get` |\n| [`params`](#reactor-req_head-params){: #reactor-req_head-params } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, appends parameters to the request query string (via `put_params` step) |\n| [`path_params_style`](#reactor-req_head-path_params_style){: #reactor-req_head-path_params_style } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | How path params are expressed (via `put_path_params` step) |\n| [`path_params`](#reactor-req_head-path_params){: #reactor-req_head-path_params } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, uses a templated request path (via `put_path_params` step) |\n| [`plug`](#reactor-req_head-plug){: #reactor-req_head-plug } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, calls the given plug instead of making an HTTP request over the network (via `run_plug` step) |\n| [`pool_timeout`](#reactor-req_head-pool_timeout){: #reactor-req_head-pool_timeout } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Pool checkout timeout in milliseconds, defaults to `5000` |\n| [`raw`](#reactor-req_head-raw){: #reactor-req_head-raw } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, disables automatic body decompression (`decompress_body` step) and decoding (`decode_body` step) |\n| [`receive_timeout`](#reactor-req_head-receive_timeout){: #reactor-req_head-receive_timeout } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Socket receive timeout in milliseconds, defaults to `15_000` |\n| [`redirect_trusted`](#reactor-req_head-redirect_trusted){: #reactor-req_head-redirect_trusted } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | By default, authorization credentials are only sent on redirects with the same host, scheme and port. If `:redirect_trusted` is set to `true`, credentials will be sent to any host |\n| [`redirect`](#reactor-req_head-redirect){: #reactor-req_head-redirect } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `false`, disables automatic response redirects |\n| [`request`](#reactor-req_head-request){: #reactor-req_head-request } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A previously built request |\n| [`retry_delay`](#reactor-req_head-retry_delay){: #reactor-req_head-retry_delay } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If not set, which is the default, the retry delay is determined by the value of retry-delay header on HTTP 429/503 responses. If the header is not set, the default delay follows a simple exponential backoff: 1s, 2s, 4s, 8s, ... |\n| [`retry_log_level`](#reactor-req_head-retry_log_level){: #reactor-req_head-retry_log_level } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The log level to emit retry logs at. Can also be set to `false` to disable logging these messages. Defaults to `:warning` |\n| [`retry`](#reactor-req_head-retry){: #reactor-req_head-retry } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | One of `:safe_transient` (default), `:transient`, `fun` or `false` |\n| [`unix_socket`](#reactor-req_head-unix_socket){: #reactor-req_head-unix_socket } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, connect through the given UNIX domain socket |\n| [`url`](#reactor-req_head-url){: #reactor-req_head-url } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request URL |","ref":"dsl-reactor-req.html#options-8"},{"type":"extras","title":"reactor.req_head.wait_for - Reactor.Req","doc":"```elixir\nwait_for names\n```\n\n\nWait for the named step to complete before allowing this one to start.\n\nDesugars to `argument :_, result(step_to_wait_for)`","ref":"dsl-reactor-req.html#reactor-req_head-wait_for"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nwait_for :create_user\n```","ref":"dsl-reactor-req.html#examples-6"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`names`](#reactor-req_head-wait_for-names){: #reactor-req_head-wait_for-names .spark-required} | `atom \\| list(atom)` |  | The name of the step to wait for. |","ref":"dsl-reactor-req.html#arguments-9"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_head-wait_for-description){: #reactor-req_head-wait_for-description } | `String.t` |  | An optional description. |","ref":"dsl-reactor-req.html#options-9"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.WaitFor`","ref":"dsl-reactor-req.html#introspection-8"},{"type":"extras","title":"reactor.req_head.guard - Reactor.Req","doc":"```elixir\nguard fun\n```\n\n\nProvides a flexible method for conditionally executing a step, or replacing it's result.\n\nExpects a two arity function which takes the step's arguments and context and returns one of the following:\n\n- `:cont` - the guard has passed.\n- `{:halt, result}` - the guard has failed - instead of executing the step use the provided result.","ref":"dsl-reactor-req.html#reactor-req_head-guard"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nstep :read_file_via_cache do\n  argument :path, input(:path)\n  run &File.read(&1.path)\n  guard fn %{path: path}, %{cache: cache} ->\n    case Cache.get(cache, path) do\n      {:ok, content} -> {:halt, {:ok, content}}\n      _ -> :cont\n    end\n  end\nend\n\n```","ref":"dsl-reactor-req.html#examples-7"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`fun`](#reactor-req_head-guard-fun){: #reactor-req_head-guard-fun .spark-required} | `(any, any -> any) \\| mfa` |  | The guard function. |","ref":"dsl-reactor-req.html#arguments-10"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_head-guard-description){: #reactor-req_head-guard-description } | `String.t` |  | An optional description of the guard. |","ref":"dsl-reactor-req.html#options-10"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.Guard`","ref":"dsl-reactor-req.html#introspection-9"},{"type":"extras","title":"reactor.req_head.where - Reactor.Req","doc":"```elixir\nwhere predicate\n```\n\n\nOnly execute the surrounding step if the predicate function returns true.\n\nThis is a simple version of `guard` which provides more flexibility at the cost of complexity.","ref":"dsl-reactor-req.html#reactor-req_head-where"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nstep :read_file do\n  argument :path, input(:path)\n  run &File.read(&1.path)\n  where &File.exists?(&1.path)\nend\n\n```","ref":"dsl-reactor-req.html#examples-8"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`predicate`](#reactor-req_head-where-predicate){: #reactor-req_head-where-predicate .spark-required} | `(any -> any) \\| mfa \\| (any, any -> any) \\| mfa` |  | Provide a function which takes the step arguments and optionally the context and returns a boolean value. |","ref":"dsl-reactor-req.html#arguments-11"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_head-where-description){: #reactor-req_head-where-description } | `String.t` |  | An optional description of the guard. |","ref":"dsl-reactor-req.html#options-11"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.Where`","ref":"dsl-reactor-req.html#introspection-10"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Req.Dsl.Head`","ref":"dsl-reactor-req.html#introspection-11"},{"type":"extras","title":"reactor.req_merge - Reactor.Req","doc":"```elixir\nreq_merge name\n```\n\n\nCreates a new request using `Req.merge/2`\n\nNote that Reactor doesn't validate any options - it simply passes them to the underlying `Req` function and assumes it will validate them.","ref":"dsl-reactor-req.html#reactor-req_merge"},{"type":"extras","title":"Nested DSLs - Reactor.Req","doc":"* [wait_for](#reactor-req_merge-wait_for)\n * [guard](#reactor-req_merge-guard)\n * [where](#reactor-req_merge-where)","ref":"dsl-reactor-req.html#nested-dsls-3"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`name`](#reactor-req_merge-name){: #reactor-req_merge-name .spark-required} | `atom` |  | A unique name for the step. Used when choosing the return value of the Reactor and for arguments into other steps |","ref":"dsl-reactor-req.html#arguments-12"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`adapter`](#reactor-req_merge-adapter){: #reactor-req_merge-adapter } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Adapter to use to make the actual HTTP request |\n| [`auth`](#reactor-req_merge-auth){: #reactor-req_merge-auth } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Sets request authentication |\n| [`aws_sigv4`](#reactor-req_merge-aws_sigv4){: #reactor-req_merge-aws_sigv4 } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, the AWS options to sign request |\n| [`base_url`](#reactor-req_merge-base_url){: #reactor-req_merge-base_url } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, the request URL is prepended with this base URL |\n| [`body`](#reactor-req_merge-body){: #reactor-req_merge-body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request body |\n| [`cache_dir`](#reactor-req_merge-cache_dir){: #reactor-req_merge-cache_dir } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The directory to store the cache |\n| [`cache`](#reactor-req_merge-cache){: #reactor-req_merge-cache } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If `true`, performs HTTP caching |\n| [`compress_body`](#reactor-req_merge-compress_body){: #reactor-req_merge-compress_body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, compresses the request body using gzip |\n| [`connect_options`](#reactor-req_merge-connect_options){: #reactor-req_merge-connect_options } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Dynamically starts (or re-uses already started) Finch pool with the given connection options |\n| [`decode_body`](#reactor-req_merge-decode_body){: #reactor-req_merge-decode_body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `false`, disables automatic response body decoding |\n| [`decode_json`](#reactor-req_merge-decode_json){: #reactor-req_merge-decode_json } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Options to pass to `Jason.decode!/2` |\n| [`finch_private`](#reactor-req_merge-finch_private){: #reactor-req_merge-finch_private } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A map or keyword list of private metadata to add to the Finch request |\n| [`finch_request`](#reactor-req_merge-finch_request){: #reactor-req_merge-finch_request } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A function that executes the Finch request, defaults to using `Finch.request/3` |\n| [`finch`](#reactor-req_merge-finch){: #reactor-req_merge-finch } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The Finch pool to use. Defaults to pool automatically started by `Req` |\n| [`form_multipart`](#reactor-req_merge-form_multipart){: #reactor-req_merge-form_multipart } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as `multipart/form-data` |\n| [`form`](#reactor-req_merge-form){: #reactor-req_merge-form } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as `application/x-www-form-urlencoded` |\n| [`headers`](#reactor-req_merge-headers){: #reactor-req_merge-headers } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request headers as a `{key, value}` enumerable (e.g. map, keyword list) |\n| [`http_errors`](#reactor-req_merge-http_errors){: #reactor-req_merge-http_errors } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | How to manage 4xx and 5xx responses |\n| [`inet6`](#reactor-req_merge-inet6){: #reactor-req_merge-inet6 } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, uses IPv6 |\n| [`into`](#reactor-req_merge-into){: #reactor-req_merge-into } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Where to send the response body |\n| [`json`](#reactor-req_merge-json){: #reactor-req_merge-json } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as JSON |\n| [`max_redirects`](#reactor-req_merge-max_redirects){: #reactor-req_merge-max_redirects } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The maximum number of redirects, defaults to `10` |\n| [`max_retries`](#reactor-req_merge-max_retries){: #reactor-req_merge-max_retries } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Maximum number of retry attempts, defaults to `3` (for a total of `4` requests to the server, including the initial one) |\n| [`method`](#reactor-req_merge-method){: #reactor-req_merge-method } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request method, defaults to `:get` |\n| [`params`](#reactor-req_merge-params){: #reactor-req_merge-params } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, appends parameters to the request query string (via `put_params` step) |\n| [`path_params_style`](#reactor-req_merge-path_params_style){: #reactor-req_merge-path_params_style } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | How path params are expressed (via `put_path_params` step) |\n| [`path_params`](#reactor-req_merge-path_params){: #reactor-req_merge-path_params } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, uses a templated request path (via `put_path_params` step) |\n| [`plug`](#reactor-req_merge-plug){: #reactor-req_merge-plug } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, calls the given plug instead of making an HTTP request over the network (via `run_plug` step) |\n| [`pool_timeout`](#reactor-req_merge-pool_timeout){: #reactor-req_merge-pool_timeout } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Pool checkout timeout in milliseconds, defaults to `5000` |\n| [`raw`](#reactor-req_merge-raw){: #reactor-req_merge-raw } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, disables automatic body decompression (`decompress_body` step) and decoding (`decode_body` step) |\n| [`receive_timeout`](#reactor-req_merge-receive_timeout){: #reactor-req_merge-receive_timeout } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Socket receive timeout in milliseconds, defaults to `15_000` |\n| [`redirect_trusted`](#reactor-req_merge-redirect_trusted){: #reactor-req_merge-redirect_trusted } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | By default, authorization credentials are only sent on redirects with the same host, scheme and port. If `:redirect_trusted` is set to `true`, credentials will be sent to any host |\n| [`redirect`](#reactor-req_merge-redirect){: #reactor-req_merge-redirect } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `false`, disables automatic response redirects |\n| [`request`](#reactor-req_merge-request){: #reactor-req_merge-request } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A previously built request |\n| [`retry_delay`](#reactor-req_merge-retry_delay){: #reactor-req_merge-retry_delay } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If not set, which is the default, the retry delay is determined by the value of retry-delay header on HTTP 429/503 responses. If the header is not set, the default delay follows a simple exponential backoff: 1s, 2s, 4s, 8s, ... |\n| [`retry_log_level`](#reactor-req_merge-retry_log_level){: #reactor-req_merge-retry_log_level } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The log level to emit retry logs at. Can also be set to `false` to disable logging these messages. Defaults to `:warning` |\n| [`retry`](#reactor-req_merge-retry){: #reactor-req_merge-retry } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | One of `:safe_transient` (default), `:transient`, `fun` or `false` |\n| [`unix_socket`](#reactor-req_merge-unix_socket){: #reactor-req_merge-unix_socket } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, connect through the given UNIX domain socket |\n| [`url`](#reactor-req_merge-url){: #reactor-req_merge-url } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request URL |","ref":"dsl-reactor-req.html#options-12"},{"type":"extras","title":"reactor.req_merge.wait_for - Reactor.Req","doc":"```elixir\nwait_for names\n```\n\n\nWait for the named step to complete before allowing this one to start.\n\nDesugars to `argument :_, result(step_to_wait_for)`","ref":"dsl-reactor-req.html#reactor-req_merge-wait_for"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nwait_for :create_user\n```","ref":"dsl-reactor-req.html#examples-9"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`names`](#reactor-req_merge-wait_for-names){: #reactor-req_merge-wait_for-names .spark-required} | `atom \\| list(atom)` |  | The name of the step to wait for. |","ref":"dsl-reactor-req.html#arguments-13"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_merge-wait_for-description){: #reactor-req_merge-wait_for-description } | `String.t` |  | An optional description. |","ref":"dsl-reactor-req.html#options-13"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.WaitFor`","ref":"dsl-reactor-req.html#introspection-12"},{"type":"extras","title":"reactor.req_merge.guard - Reactor.Req","doc":"```elixir\nguard fun\n```\n\n\nProvides a flexible method for conditionally executing a step, or replacing it's result.\n\nExpects a two arity function which takes the step's arguments and context and returns one of the following:\n\n- `:cont` - the guard has passed.\n- `{:halt, result}` - the guard has failed - instead of executing the step use the provided result.","ref":"dsl-reactor-req.html#reactor-req_merge-guard"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nstep :read_file_via_cache do\n  argument :path, input(:path)\n  run &File.read(&1.path)\n  guard fn %{path: path}, %{cache: cache} ->\n    case Cache.get(cache, path) do\n      {:ok, content} -> {:halt, {:ok, content}}\n      _ -> :cont\n    end\n  end\nend\n\n```","ref":"dsl-reactor-req.html#examples-10"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`fun`](#reactor-req_merge-guard-fun){: #reactor-req_merge-guard-fun .spark-required} | `(any, any -> any) \\| mfa` |  | The guard function. |","ref":"dsl-reactor-req.html#arguments-14"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_merge-guard-description){: #reactor-req_merge-guard-description } | `String.t` |  | An optional description of the guard. |","ref":"dsl-reactor-req.html#options-14"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.Guard`","ref":"dsl-reactor-req.html#introspection-13"},{"type":"extras","title":"reactor.req_merge.where - Reactor.Req","doc":"```elixir\nwhere predicate\n```\n\n\nOnly execute the surrounding step if the predicate function returns true.\n\nThis is a simple version of `guard` which provides more flexibility at the cost of complexity.","ref":"dsl-reactor-req.html#reactor-req_merge-where"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nstep :read_file do\n  argument :path, input(:path)\n  run &File.read(&1.path)\n  where &File.exists?(&1.path)\nend\n\n```","ref":"dsl-reactor-req.html#examples-11"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`predicate`](#reactor-req_merge-where-predicate){: #reactor-req_merge-where-predicate .spark-required} | `(any -> any) \\| mfa \\| (any, any -> any) \\| mfa` |  | Provide a function which takes the step arguments and optionally the context and returns a boolean value. |","ref":"dsl-reactor-req.html#arguments-15"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_merge-where-description){: #reactor-req_merge-where-description } | `String.t` |  | An optional description of the guard. |","ref":"dsl-reactor-req.html#options-15"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.Where`","ref":"dsl-reactor-req.html#introspection-14"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Req.Dsl.Merge`","ref":"dsl-reactor-req.html#introspection-15"},{"type":"extras","title":"reactor.req_new - Reactor.Req","doc":"```elixir\nreq_new name\n```\n\n\nCreates a new request using `Req.new/1`\n\nNote that Reactor doesn't validate any options - it simply passes them to the underlying `Req` function and assumes it will validate them.","ref":"dsl-reactor-req.html#reactor-req_new"},{"type":"extras","title":"Nested DSLs - Reactor.Req","doc":"* [wait_for](#reactor-req_new-wait_for)\n * [guard](#reactor-req_new-guard)\n * [where](#reactor-req_new-where)","ref":"dsl-reactor-req.html#nested-dsls-4"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`name`](#reactor-req_new-name){: #reactor-req_new-name .spark-required} | `atom` |  | A unique name for the step. Used when choosing the return value of the Reactor and for arguments into other steps |","ref":"dsl-reactor-req.html#arguments-16"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`adapter`](#reactor-req_new-adapter){: #reactor-req_new-adapter } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Adapter to use to make the actual HTTP request |\n| [`auth`](#reactor-req_new-auth){: #reactor-req_new-auth } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Sets request authentication |\n| [`aws_sigv4`](#reactor-req_new-aws_sigv4){: #reactor-req_new-aws_sigv4 } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, the AWS options to sign request |\n| [`base_url`](#reactor-req_new-base_url){: #reactor-req_new-base_url } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, the request URL is prepended with this base URL |\n| [`body`](#reactor-req_new-body){: #reactor-req_new-body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request body |\n| [`cache_dir`](#reactor-req_new-cache_dir){: #reactor-req_new-cache_dir } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The directory to store the cache |\n| [`cache`](#reactor-req_new-cache){: #reactor-req_new-cache } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If `true`, performs HTTP caching |\n| [`compress_body`](#reactor-req_new-compress_body){: #reactor-req_new-compress_body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, compresses the request body using gzip |\n| [`connect_options`](#reactor-req_new-connect_options){: #reactor-req_new-connect_options } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Dynamically starts (or re-uses already started) Finch pool with the given connection options |\n| [`decode_body`](#reactor-req_new-decode_body){: #reactor-req_new-decode_body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `false`, disables automatic response body decoding |\n| [`decode_json`](#reactor-req_new-decode_json){: #reactor-req_new-decode_json } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Options to pass to `Jason.decode!/2` |\n| [`finch_private`](#reactor-req_new-finch_private){: #reactor-req_new-finch_private } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A map or keyword list of private metadata to add to the Finch request |\n| [`finch_request`](#reactor-req_new-finch_request){: #reactor-req_new-finch_request } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A function that executes the Finch request, defaults to using `Finch.request/3` |\n| [`finch`](#reactor-req_new-finch){: #reactor-req_new-finch } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The Finch pool to use. Defaults to pool automatically started by `Req` |\n| [`form_multipart`](#reactor-req_new-form_multipart){: #reactor-req_new-form_multipart } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as `multipart/form-data` |\n| [`form`](#reactor-req_new-form){: #reactor-req_new-form } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as `application/x-www-form-urlencoded` |\n| [`headers`](#reactor-req_new-headers){: #reactor-req_new-headers } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request headers as a `{key, value}` enumerable (e.g. map, keyword list) |\n| [`http_errors`](#reactor-req_new-http_errors){: #reactor-req_new-http_errors } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | How to manage 4xx and 5xx responses |\n| [`inet6`](#reactor-req_new-inet6){: #reactor-req_new-inet6 } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, uses IPv6 |\n| [`into`](#reactor-req_new-into){: #reactor-req_new-into } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Where to send the response body |\n| [`json`](#reactor-req_new-json){: #reactor-req_new-json } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as JSON |\n| [`max_redirects`](#reactor-req_new-max_redirects){: #reactor-req_new-max_redirects } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The maximum number of redirects, defaults to `10` |\n| [`max_retries`](#reactor-req_new-max_retries){: #reactor-req_new-max_retries } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Maximum number of retry attempts, defaults to `3` (for a total of `4` requests to the server, including the initial one) |\n| [`method`](#reactor-req_new-method){: #reactor-req_new-method } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request method, defaults to `:get` |\n| [`params`](#reactor-req_new-params){: #reactor-req_new-params } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, appends parameters to the request query string (via `put_params` step) |\n| [`path_params_style`](#reactor-req_new-path_params_style){: #reactor-req_new-path_params_style } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | How path params are expressed (via `put_path_params` step) |\n| [`path_params`](#reactor-req_new-path_params){: #reactor-req_new-path_params } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, uses a templated request path (via `put_path_params` step) |\n| [`plug`](#reactor-req_new-plug){: #reactor-req_new-plug } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, calls the given plug instead of making an HTTP request over the network (via `run_plug` step) |\n| [`pool_timeout`](#reactor-req_new-pool_timeout){: #reactor-req_new-pool_timeout } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Pool checkout timeout in milliseconds, defaults to `5000` |\n| [`raw`](#reactor-req_new-raw){: #reactor-req_new-raw } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, disables automatic body decompression (`decompress_body` step) and decoding (`decode_body` step) |\n| [`receive_timeout`](#reactor-req_new-receive_timeout){: #reactor-req_new-receive_timeout } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Socket receive timeout in milliseconds, defaults to `15_000` |\n| [`redirect_trusted`](#reactor-req_new-redirect_trusted){: #reactor-req_new-redirect_trusted } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | By default, authorization credentials are only sent on redirects with the same host, scheme and port. If `:redirect_trusted` is set to `true`, credentials will be sent to any host |\n| [`redirect`](#reactor-req_new-redirect){: #reactor-req_new-redirect } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `false`, disables automatic response redirects |\n| [`request`](#reactor-req_new-request){: #reactor-req_new-request } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A previously built request |\n| [`retry_delay`](#reactor-req_new-retry_delay){: #reactor-req_new-retry_delay } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If not set, which is the default, the retry delay is determined by the value of retry-delay header on HTTP 429/503 responses. If the header is not set, the default delay follows a simple exponential backoff: 1s, 2s, 4s, 8s, ... |\n| [`retry_log_level`](#reactor-req_new-retry_log_level){: #reactor-req_new-retry_log_level } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The log level to emit retry logs at. Can also be set to `false` to disable logging these messages. Defaults to `:warning` |\n| [`retry`](#reactor-req_new-retry){: #reactor-req_new-retry } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | One of `:safe_transient` (default), `:transient`, `fun` or `false` |\n| [`unix_socket`](#reactor-req_new-unix_socket){: #reactor-req_new-unix_socket } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, connect through the given UNIX domain socket |\n| [`url`](#reactor-req_new-url){: #reactor-req_new-url } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request URL |","ref":"dsl-reactor-req.html#options-16"},{"type":"extras","title":"reactor.req_new.wait_for - Reactor.Req","doc":"```elixir\nwait_for names\n```\n\n\nWait for the named step to complete before allowing this one to start.\n\nDesugars to `argument :_, result(step_to_wait_for)`","ref":"dsl-reactor-req.html#reactor-req_new-wait_for"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nwait_for :create_user\n```","ref":"dsl-reactor-req.html#examples-12"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`names`](#reactor-req_new-wait_for-names){: #reactor-req_new-wait_for-names .spark-required} | `atom \\| list(atom)` |  | The name of the step to wait for. |","ref":"dsl-reactor-req.html#arguments-17"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_new-wait_for-description){: #reactor-req_new-wait_for-description } | `String.t` |  | An optional description. |","ref":"dsl-reactor-req.html#options-17"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.WaitFor`","ref":"dsl-reactor-req.html#introspection-16"},{"type":"extras","title":"reactor.req_new.guard - Reactor.Req","doc":"```elixir\nguard fun\n```\n\n\nProvides a flexible method for conditionally executing a step, or replacing it's result.\n\nExpects a two arity function which takes the step's arguments and context and returns one of the following:\n\n- `:cont` - the guard has passed.\n- `{:halt, result}` - the guard has failed - instead of executing the step use the provided result.","ref":"dsl-reactor-req.html#reactor-req_new-guard"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nstep :read_file_via_cache do\n  argument :path, input(:path)\n  run &File.read(&1.path)\n  guard fn %{path: path}, %{cache: cache} ->\n    case Cache.get(cache, path) do\n      {:ok, content} -> {:halt, {:ok, content}}\n      _ -> :cont\n    end\n  end\nend\n\n```","ref":"dsl-reactor-req.html#examples-13"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`fun`](#reactor-req_new-guard-fun){: #reactor-req_new-guard-fun .spark-required} | `(any, any -> any) \\| mfa` |  | The guard function. |","ref":"dsl-reactor-req.html#arguments-18"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_new-guard-description){: #reactor-req_new-guard-description } | `String.t` |  | An optional description of the guard. |","ref":"dsl-reactor-req.html#options-18"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.Guard`","ref":"dsl-reactor-req.html#introspection-17"},{"type":"extras","title":"reactor.req_new.where - Reactor.Req","doc":"```elixir\nwhere predicate\n```\n\n\nOnly execute the surrounding step if the predicate function returns true.\n\nThis is a simple version of `guard` which provides more flexibility at the cost of complexity.","ref":"dsl-reactor-req.html#reactor-req_new-where"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nstep :read_file do\n  argument :path, input(:path)\n  run &File.read(&1.path)\n  where &File.exists?(&1.path)\nend\n\n```","ref":"dsl-reactor-req.html#examples-14"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`predicate`](#reactor-req_new-where-predicate){: #reactor-req_new-where-predicate .spark-required} | `(any -> any) \\| mfa \\| (any, any -> any) \\| mfa` |  | Provide a function which takes the step arguments and optionally the context and returns a boolean value. |","ref":"dsl-reactor-req.html#arguments-19"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_new-where-description){: #reactor-req_new-where-description } | `String.t` |  | An optional description of the guard. |","ref":"dsl-reactor-req.html#options-19"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.Where`","ref":"dsl-reactor-req.html#introspection-18"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Req.Dsl.New`","ref":"dsl-reactor-req.html#introspection-19"},{"type":"extras","title":"reactor.req_patch - Reactor.Req","doc":"```elixir\nreq_patch name\n```\n\n\nPerforms a request using `Req.patch/2`.\n\nNote that Reactor doesn't validate any options - it simply passes them to the underlying `Req` function and assumes it will validate them.","ref":"dsl-reactor-req.html#reactor-req_patch"},{"type":"extras","title":"Nested DSLs - Reactor.Req","doc":"* [wait_for](#reactor-req_patch-wait_for)\n * [guard](#reactor-req_patch-guard)\n * [where](#reactor-req_patch-where)","ref":"dsl-reactor-req.html#nested-dsls-5"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`name`](#reactor-req_patch-name){: #reactor-req_patch-name .spark-required} | `atom` |  | A unique name for the step. Used when choosing the return value of the Reactor and for arguments into other steps |","ref":"dsl-reactor-req.html#arguments-20"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`adapter`](#reactor-req_patch-adapter){: #reactor-req_patch-adapter } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Adapter to use to make the actual HTTP request |\n| [`auth`](#reactor-req_patch-auth){: #reactor-req_patch-auth } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Sets request authentication |\n| [`aws_sigv4`](#reactor-req_patch-aws_sigv4){: #reactor-req_patch-aws_sigv4 } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, the AWS options to sign request |\n| [`base_url`](#reactor-req_patch-base_url){: #reactor-req_patch-base_url } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, the request URL is prepended with this base URL |\n| [`body`](#reactor-req_patch-body){: #reactor-req_patch-body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request body |\n| [`cache_dir`](#reactor-req_patch-cache_dir){: #reactor-req_patch-cache_dir } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The directory to store the cache |\n| [`cache`](#reactor-req_patch-cache){: #reactor-req_patch-cache } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If `true`, performs HTTP caching |\n| [`compress_body`](#reactor-req_patch-compress_body){: #reactor-req_patch-compress_body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, compresses the request body using gzip |\n| [`connect_options`](#reactor-req_patch-connect_options){: #reactor-req_patch-connect_options } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Dynamically starts (or re-uses already started) Finch pool with the given connection options |\n| [`decode_body`](#reactor-req_patch-decode_body){: #reactor-req_patch-decode_body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `false`, disables automatic response body decoding |\n| [`decode_json`](#reactor-req_patch-decode_json){: #reactor-req_patch-decode_json } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Options to pass to `Jason.decode!/2` |\n| [`finch_private`](#reactor-req_patch-finch_private){: #reactor-req_patch-finch_private } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A map or keyword list of private metadata to add to the Finch request |\n| [`finch_request`](#reactor-req_patch-finch_request){: #reactor-req_patch-finch_request } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A function that executes the Finch request, defaults to using `Finch.request/3` |\n| [`finch`](#reactor-req_patch-finch){: #reactor-req_patch-finch } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The Finch pool to use. Defaults to pool automatically started by `Req` |\n| [`form_multipart`](#reactor-req_patch-form_multipart){: #reactor-req_patch-form_multipart } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as `multipart/form-data` |\n| [`form`](#reactor-req_patch-form){: #reactor-req_patch-form } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as `application/x-www-form-urlencoded` |\n| [`headers`](#reactor-req_patch-headers){: #reactor-req_patch-headers } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request headers as a `{key, value}` enumerable (e.g. map, keyword list) |\n| [`http_errors`](#reactor-req_patch-http_errors){: #reactor-req_patch-http_errors } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | How to manage 4xx and 5xx responses |\n| [`inet6`](#reactor-req_patch-inet6){: #reactor-req_patch-inet6 } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, uses IPv6 |\n| [`into`](#reactor-req_patch-into){: #reactor-req_patch-into } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Where to send the response body |\n| [`json`](#reactor-req_patch-json){: #reactor-req_patch-json } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as JSON |\n| [`max_redirects`](#reactor-req_patch-max_redirects){: #reactor-req_patch-max_redirects } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The maximum number of redirects, defaults to `10` |\n| [`max_retries`](#reactor-req_patch-max_retries){: #reactor-req_patch-max_retries } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Maximum number of retry attempts, defaults to `3` (for a total of `4` requests to the server, including the initial one) |\n| [`method`](#reactor-req_patch-method){: #reactor-req_patch-method } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request method, defaults to `:get` |\n| [`params`](#reactor-req_patch-params){: #reactor-req_patch-params } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, appends parameters to the request query string (via `put_params` step) |\n| [`path_params_style`](#reactor-req_patch-path_params_style){: #reactor-req_patch-path_params_style } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | How path params are expressed (via `put_path_params` step) |\n| [`path_params`](#reactor-req_patch-path_params){: #reactor-req_patch-path_params } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, uses a templated request path (via `put_path_params` step) |\n| [`plug`](#reactor-req_patch-plug){: #reactor-req_patch-plug } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, calls the given plug instead of making an HTTP request over the network (via `run_plug` step) |\n| [`pool_timeout`](#reactor-req_patch-pool_timeout){: #reactor-req_patch-pool_timeout } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Pool checkout timeout in milliseconds, defaults to `5000` |\n| [`raw`](#reactor-req_patch-raw){: #reactor-req_patch-raw } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, disables automatic body decompression (`decompress_body` step) and decoding (`decode_body` step) |\n| [`receive_timeout`](#reactor-req_patch-receive_timeout){: #reactor-req_patch-receive_timeout } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Socket receive timeout in milliseconds, defaults to `15_000` |\n| [`redirect_trusted`](#reactor-req_patch-redirect_trusted){: #reactor-req_patch-redirect_trusted } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | By default, authorization credentials are only sent on redirects with the same host, scheme and port. If `:redirect_trusted` is set to `true`, credentials will be sent to any host |\n| [`redirect`](#reactor-req_patch-redirect){: #reactor-req_patch-redirect } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `false`, disables automatic response redirects |\n| [`request`](#reactor-req_patch-request){: #reactor-req_patch-request } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A previously built request |\n| [`retry_delay`](#reactor-req_patch-retry_delay){: #reactor-req_patch-retry_delay } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If not set, which is the default, the retry delay is determined by the value of retry-delay header on HTTP 429/503 responses. If the header is not set, the default delay follows a simple exponential backoff: 1s, 2s, 4s, 8s, ... |\n| [`retry_log_level`](#reactor-req_patch-retry_log_level){: #reactor-req_patch-retry_log_level } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The log level to emit retry logs at. Can also be set to `false` to disable logging these messages. Defaults to `:warning` |\n| [`retry`](#reactor-req_patch-retry){: #reactor-req_patch-retry } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | One of `:safe_transient` (default), `:transient`, `fun` or `false` |\n| [`unix_socket`](#reactor-req_patch-unix_socket){: #reactor-req_patch-unix_socket } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, connect through the given UNIX domain socket |\n| [`url`](#reactor-req_patch-url){: #reactor-req_patch-url } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request URL |","ref":"dsl-reactor-req.html#options-20"},{"type":"extras","title":"reactor.req_patch.wait_for - Reactor.Req","doc":"```elixir\nwait_for names\n```\n\n\nWait for the named step to complete before allowing this one to start.\n\nDesugars to `argument :_, result(step_to_wait_for)`","ref":"dsl-reactor-req.html#reactor-req_patch-wait_for"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nwait_for :create_user\n```","ref":"dsl-reactor-req.html#examples-15"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`names`](#reactor-req_patch-wait_for-names){: #reactor-req_patch-wait_for-names .spark-required} | `atom \\| list(atom)` |  | The name of the step to wait for. |","ref":"dsl-reactor-req.html#arguments-21"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_patch-wait_for-description){: #reactor-req_patch-wait_for-description } | `String.t` |  | An optional description. |","ref":"dsl-reactor-req.html#options-21"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.WaitFor`","ref":"dsl-reactor-req.html#introspection-20"},{"type":"extras","title":"reactor.req_patch.guard - Reactor.Req","doc":"```elixir\nguard fun\n```\n\n\nProvides a flexible method for conditionally executing a step, or replacing it's result.\n\nExpects a two arity function which takes the step's arguments and context and returns one of the following:\n\n- `:cont` - the guard has passed.\n- `{:halt, result}` - the guard has failed - instead of executing the step use the provided result.","ref":"dsl-reactor-req.html#reactor-req_patch-guard"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nstep :read_file_via_cache do\n  argument :path, input(:path)\n  run &File.read(&1.path)\n  guard fn %{path: path}, %{cache: cache} ->\n    case Cache.get(cache, path) do\n      {:ok, content} -> {:halt, {:ok, content}}\n      _ -> :cont\n    end\n  end\nend\n\n```","ref":"dsl-reactor-req.html#examples-16"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`fun`](#reactor-req_patch-guard-fun){: #reactor-req_patch-guard-fun .spark-required} | `(any, any -> any) \\| mfa` |  | The guard function. |","ref":"dsl-reactor-req.html#arguments-22"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_patch-guard-description){: #reactor-req_patch-guard-description } | `String.t` |  | An optional description of the guard. |","ref":"dsl-reactor-req.html#options-22"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.Guard`","ref":"dsl-reactor-req.html#introspection-21"},{"type":"extras","title":"reactor.req_patch.where - Reactor.Req","doc":"```elixir\nwhere predicate\n```\n\n\nOnly execute the surrounding step if the predicate function returns true.\n\nThis is a simple version of `guard` which provides more flexibility at the cost of complexity.","ref":"dsl-reactor-req.html#reactor-req_patch-where"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nstep :read_file do\n  argument :path, input(:path)\n  run &File.read(&1.path)\n  where &File.exists?(&1.path)\nend\n\n```","ref":"dsl-reactor-req.html#examples-17"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`predicate`](#reactor-req_patch-where-predicate){: #reactor-req_patch-where-predicate .spark-required} | `(any -> any) \\| mfa \\| (any, any -> any) \\| mfa` |  | Provide a function which takes the step arguments and optionally the context and returns a boolean value. |","ref":"dsl-reactor-req.html#arguments-23"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_patch-where-description){: #reactor-req_patch-where-description } | `String.t` |  | An optional description of the guard. |","ref":"dsl-reactor-req.html#options-23"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.Where`","ref":"dsl-reactor-req.html#introspection-22"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Req.Dsl.Patch`","ref":"dsl-reactor-req.html#introspection-23"},{"type":"extras","title":"reactor.req_post - Reactor.Req","doc":"```elixir\nreq_post name\n```\n\n\nPerforms a request using `Req.post/2`.\n\nNote that Reactor doesn't validate any options - it simply passes them to the underlying `Req` function and assumes it will validate them.","ref":"dsl-reactor-req.html#reactor-req_post"},{"type":"extras","title":"Nested DSLs - Reactor.Req","doc":"* [wait_for](#reactor-req_post-wait_for)\n * [guard](#reactor-req_post-guard)\n * [where](#reactor-req_post-where)","ref":"dsl-reactor-req.html#nested-dsls-6"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`name`](#reactor-req_post-name){: #reactor-req_post-name .spark-required} | `atom` |  | A unique name for the step. Used when choosing the return value of the Reactor and for arguments into other steps |","ref":"dsl-reactor-req.html#arguments-24"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`adapter`](#reactor-req_post-adapter){: #reactor-req_post-adapter } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Adapter to use to make the actual HTTP request |\n| [`auth`](#reactor-req_post-auth){: #reactor-req_post-auth } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Sets request authentication |\n| [`aws_sigv4`](#reactor-req_post-aws_sigv4){: #reactor-req_post-aws_sigv4 } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, the AWS options to sign request |\n| [`base_url`](#reactor-req_post-base_url){: #reactor-req_post-base_url } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, the request URL is prepended with this base URL |\n| [`body`](#reactor-req_post-body){: #reactor-req_post-body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request body |\n| [`cache_dir`](#reactor-req_post-cache_dir){: #reactor-req_post-cache_dir } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The directory to store the cache |\n| [`cache`](#reactor-req_post-cache){: #reactor-req_post-cache } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If `true`, performs HTTP caching |\n| [`compress_body`](#reactor-req_post-compress_body){: #reactor-req_post-compress_body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, compresses the request body using gzip |\n| [`connect_options`](#reactor-req_post-connect_options){: #reactor-req_post-connect_options } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Dynamically starts (or re-uses already started) Finch pool with the given connection options |\n| [`decode_body`](#reactor-req_post-decode_body){: #reactor-req_post-decode_body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `false`, disables automatic response body decoding |\n| [`decode_json`](#reactor-req_post-decode_json){: #reactor-req_post-decode_json } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Options to pass to `Jason.decode!/2` |\n| [`finch_private`](#reactor-req_post-finch_private){: #reactor-req_post-finch_private } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A map or keyword list of private metadata to add to the Finch request |\n| [`finch_request`](#reactor-req_post-finch_request){: #reactor-req_post-finch_request } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A function that executes the Finch request, defaults to using `Finch.request/3` |\n| [`finch`](#reactor-req_post-finch){: #reactor-req_post-finch } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The Finch pool to use. Defaults to pool automatically started by `Req` |\n| [`form_multipart`](#reactor-req_post-form_multipart){: #reactor-req_post-form_multipart } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as `multipart/form-data` |\n| [`form`](#reactor-req_post-form){: #reactor-req_post-form } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as `application/x-www-form-urlencoded` |\n| [`headers`](#reactor-req_post-headers){: #reactor-req_post-headers } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request headers as a `{key, value}` enumerable (e.g. map, keyword list) |\n| [`http_errors`](#reactor-req_post-http_errors){: #reactor-req_post-http_errors } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | How to manage 4xx and 5xx responses |\n| [`inet6`](#reactor-req_post-inet6){: #reactor-req_post-inet6 } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, uses IPv6 |\n| [`into`](#reactor-req_post-into){: #reactor-req_post-into } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Where to send the response body |\n| [`json`](#reactor-req_post-json){: #reactor-req_post-json } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as JSON |\n| [`max_redirects`](#reactor-req_post-max_redirects){: #reactor-req_post-max_redirects } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The maximum number of redirects, defaults to `10` |\n| [`max_retries`](#reactor-req_post-max_retries){: #reactor-req_post-max_retries } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Maximum number of retry attempts, defaults to `3` (for a total of `4` requests to the server, including the initial one) |\n| [`method`](#reactor-req_post-method){: #reactor-req_post-method } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request method, defaults to `:get` |\n| [`params`](#reactor-req_post-params){: #reactor-req_post-params } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, appends parameters to the request query string (via `put_params` step) |\n| [`path_params_style`](#reactor-req_post-path_params_style){: #reactor-req_post-path_params_style } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | How path params are expressed (via `put_path_params` step) |\n| [`path_params`](#reactor-req_post-path_params){: #reactor-req_post-path_params } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, uses a templated request path (via `put_path_params` step) |\n| [`plug`](#reactor-req_post-plug){: #reactor-req_post-plug } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, calls the given plug instead of making an HTTP request over the network (via `run_plug` step) |\n| [`pool_timeout`](#reactor-req_post-pool_timeout){: #reactor-req_post-pool_timeout } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Pool checkout timeout in milliseconds, defaults to `5000` |\n| [`raw`](#reactor-req_post-raw){: #reactor-req_post-raw } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, disables automatic body decompression (`decompress_body` step) and decoding (`decode_body` step) |\n| [`receive_timeout`](#reactor-req_post-receive_timeout){: #reactor-req_post-receive_timeout } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Socket receive timeout in milliseconds, defaults to `15_000` |\n| [`redirect_trusted`](#reactor-req_post-redirect_trusted){: #reactor-req_post-redirect_trusted } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | By default, authorization credentials are only sent on redirects with the same host, scheme and port. If `:redirect_trusted` is set to `true`, credentials will be sent to any host |\n| [`redirect`](#reactor-req_post-redirect){: #reactor-req_post-redirect } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `false`, disables automatic response redirects |\n| [`request`](#reactor-req_post-request){: #reactor-req_post-request } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A previously built request |\n| [`retry_delay`](#reactor-req_post-retry_delay){: #reactor-req_post-retry_delay } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If not set, which is the default, the retry delay is determined by the value of retry-delay header on HTTP 429/503 responses. If the header is not set, the default delay follows a simple exponential backoff: 1s, 2s, 4s, 8s, ... |\n| [`retry_log_level`](#reactor-req_post-retry_log_level){: #reactor-req_post-retry_log_level } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The log level to emit retry logs at. Can also be set to `false` to disable logging these messages. Defaults to `:warning` |\n| [`retry`](#reactor-req_post-retry){: #reactor-req_post-retry } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | One of `:safe_transient` (default), `:transient`, `fun` or `false` |\n| [`unix_socket`](#reactor-req_post-unix_socket){: #reactor-req_post-unix_socket } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, connect through the given UNIX domain socket |\n| [`url`](#reactor-req_post-url){: #reactor-req_post-url } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request URL |","ref":"dsl-reactor-req.html#options-24"},{"type":"extras","title":"reactor.req_post.wait_for - Reactor.Req","doc":"```elixir\nwait_for names\n```\n\n\nWait for the named step to complete before allowing this one to start.\n\nDesugars to `argument :_, result(step_to_wait_for)`","ref":"dsl-reactor-req.html#reactor-req_post-wait_for"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nwait_for :create_user\n```","ref":"dsl-reactor-req.html#examples-18"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`names`](#reactor-req_post-wait_for-names){: #reactor-req_post-wait_for-names .spark-required} | `atom \\| list(atom)` |  | The name of the step to wait for. |","ref":"dsl-reactor-req.html#arguments-25"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_post-wait_for-description){: #reactor-req_post-wait_for-description } | `String.t` |  | An optional description. |","ref":"dsl-reactor-req.html#options-25"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.WaitFor`","ref":"dsl-reactor-req.html#introspection-24"},{"type":"extras","title":"reactor.req_post.guard - Reactor.Req","doc":"```elixir\nguard fun\n```\n\n\nProvides a flexible method for conditionally executing a step, or replacing it's result.\n\nExpects a two arity function which takes the step's arguments and context and returns one of the following:\n\n- `:cont` - the guard has passed.\n- `{:halt, result}` - the guard has failed - instead of executing the step use the provided result.","ref":"dsl-reactor-req.html#reactor-req_post-guard"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nstep :read_file_via_cache do\n  argument :path, input(:path)\n  run &File.read(&1.path)\n  guard fn %{path: path}, %{cache: cache} ->\n    case Cache.get(cache, path) do\n      {:ok, content} -> {:halt, {:ok, content}}\n      _ -> :cont\n    end\n  end\nend\n\n```","ref":"dsl-reactor-req.html#examples-19"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`fun`](#reactor-req_post-guard-fun){: #reactor-req_post-guard-fun .spark-required} | `(any, any -> any) \\| mfa` |  | The guard function. |","ref":"dsl-reactor-req.html#arguments-26"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_post-guard-description){: #reactor-req_post-guard-description } | `String.t` |  | An optional description of the guard. |","ref":"dsl-reactor-req.html#options-26"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.Guard`","ref":"dsl-reactor-req.html#introspection-25"},{"type":"extras","title":"reactor.req_post.where - Reactor.Req","doc":"```elixir\nwhere predicate\n```\n\n\nOnly execute the surrounding step if the predicate function returns true.\n\nThis is a simple version of `guard` which provides more flexibility at the cost of complexity.","ref":"dsl-reactor-req.html#reactor-req_post-where"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nstep :read_file do\n  argument :path, input(:path)\n  run &File.read(&1.path)\n  where &File.exists?(&1.path)\nend\n\n```","ref":"dsl-reactor-req.html#examples-20"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`predicate`](#reactor-req_post-where-predicate){: #reactor-req_post-where-predicate .spark-required} | `(any -> any) \\| mfa \\| (any, any -> any) \\| mfa` |  | Provide a function which takes the step arguments and optionally the context and returns a boolean value. |","ref":"dsl-reactor-req.html#arguments-27"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_post-where-description){: #reactor-req_post-where-description } | `String.t` |  | An optional description of the guard. |","ref":"dsl-reactor-req.html#options-27"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.Where`","ref":"dsl-reactor-req.html#introspection-26"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Req.Dsl.Post`","ref":"dsl-reactor-req.html#introspection-27"},{"type":"extras","title":"reactor.req_put - Reactor.Req","doc":"```elixir\nreq_put name\n```\n\n\nPerforms a request using `Req.put/2`.\n\nNote that Reactor doesn't validate any options - it simply passes them to the underlying `Req` function and assumes it will validate them.","ref":"dsl-reactor-req.html#reactor-req_put"},{"type":"extras","title":"Nested DSLs - Reactor.Req","doc":"* [wait_for](#reactor-req_put-wait_for)\n * [guard](#reactor-req_put-guard)\n * [where](#reactor-req_put-where)","ref":"dsl-reactor-req.html#nested-dsls-7"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`name`](#reactor-req_put-name){: #reactor-req_put-name .spark-required} | `atom` |  | A unique name for the step. Used when choosing the return value of the Reactor and for arguments into other steps |","ref":"dsl-reactor-req.html#arguments-28"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`adapter`](#reactor-req_put-adapter){: #reactor-req_put-adapter } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Adapter to use to make the actual HTTP request |\n| [`auth`](#reactor-req_put-auth){: #reactor-req_put-auth } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Sets request authentication |\n| [`aws_sigv4`](#reactor-req_put-aws_sigv4){: #reactor-req_put-aws_sigv4 } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, the AWS options to sign request |\n| [`base_url`](#reactor-req_put-base_url){: #reactor-req_put-base_url } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, the request URL is prepended with this base URL |\n| [`body`](#reactor-req_put-body){: #reactor-req_put-body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request body |\n| [`cache_dir`](#reactor-req_put-cache_dir){: #reactor-req_put-cache_dir } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The directory to store the cache |\n| [`cache`](#reactor-req_put-cache){: #reactor-req_put-cache } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If `true`, performs HTTP caching |\n| [`compress_body`](#reactor-req_put-compress_body){: #reactor-req_put-compress_body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, compresses the request body using gzip |\n| [`connect_options`](#reactor-req_put-connect_options){: #reactor-req_put-connect_options } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Dynamically starts (or re-uses already started) Finch pool with the given connection options |\n| [`decode_body`](#reactor-req_put-decode_body){: #reactor-req_put-decode_body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `false`, disables automatic response body decoding |\n| [`decode_json`](#reactor-req_put-decode_json){: #reactor-req_put-decode_json } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Options to pass to `Jason.decode!/2` |\n| [`finch_private`](#reactor-req_put-finch_private){: #reactor-req_put-finch_private } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A map or keyword list of private metadata to add to the Finch request |\n| [`finch_request`](#reactor-req_put-finch_request){: #reactor-req_put-finch_request } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A function that executes the Finch request, defaults to using `Finch.request/3` |\n| [`finch`](#reactor-req_put-finch){: #reactor-req_put-finch } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The Finch pool to use. Defaults to pool automatically started by `Req` |\n| [`form_multipart`](#reactor-req_put-form_multipart){: #reactor-req_put-form_multipart } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as `multipart/form-data` |\n| [`form`](#reactor-req_put-form){: #reactor-req_put-form } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as `application/x-www-form-urlencoded` |\n| [`headers`](#reactor-req_put-headers){: #reactor-req_put-headers } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request headers as a `{key, value}` enumerable (e.g. map, keyword list) |\n| [`http_errors`](#reactor-req_put-http_errors){: #reactor-req_put-http_errors } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | How to manage 4xx and 5xx responses |\n| [`inet6`](#reactor-req_put-inet6){: #reactor-req_put-inet6 } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, uses IPv6 |\n| [`into`](#reactor-req_put-into){: #reactor-req_put-into } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Where to send the response body |\n| [`json`](#reactor-req_put-json){: #reactor-req_put-json } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as JSON |\n| [`max_redirects`](#reactor-req_put-max_redirects){: #reactor-req_put-max_redirects } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The maximum number of redirects, defaults to `10` |\n| [`max_retries`](#reactor-req_put-max_retries){: #reactor-req_put-max_retries } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Maximum number of retry attempts, defaults to `3` (for a total of `4` requests to the server, including the initial one) |\n| [`method`](#reactor-req_put-method){: #reactor-req_put-method } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request method, defaults to `:get` |\n| [`params`](#reactor-req_put-params){: #reactor-req_put-params } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, appends parameters to the request query string (via `put_params` step) |\n| [`path_params_style`](#reactor-req_put-path_params_style){: #reactor-req_put-path_params_style } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | How path params are expressed (via `put_path_params` step) |\n| [`path_params`](#reactor-req_put-path_params){: #reactor-req_put-path_params } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, uses a templated request path (via `put_path_params` step) |\n| [`plug`](#reactor-req_put-plug){: #reactor-req_put-plug } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, calls the given plug instead of making an HTTP request over the network (via `run_plug` step) |\n| [`pool_timeout`](#reactor-req_put-pool_timeout){: #reactor-req_put-pool_timeout } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Pool checkout timeout in milliseconds, defaults to `5000` |\n| [`raw`](#reactor-req_put-raw){: #reactor-req_put-raw } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, disables automatic body decompression (`decompress_body` step) and decoding (`decode_body` step) |\n| [`receive_timeout`](#reactor-req_put-receive_timeout){: #reactor-req_put-receive_timeout } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Socket receive timeout in milliseconds, defaults to `15_000` |\n| [`redirect_trusted`](#reactor-req_put-redirect_trusted){: #reactor-req_put-redirect_trusted } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | By default, authorization credentials are only sent on redirects with the same host, scheme and port. If `:redirect_trusted` is set to `true`, credentials will be sent to any host |\n| [`redirect`](#reactor-req_put-redirect){: #reactor-req_put-redirect } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `false`, disables automatic response redirects |\n| [`request`](#reactor-req_put-request){: #reactor-req_put-request } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A previously built request |\n| [`retry_delay`](#reactor-req_put-retry_delay){: #reactor-req_put-retry_delay } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If not set, which is the default, the retry delay is determined by the value of retry-delay header on HTTP 429/503 responses. If the header is not set, the default delay follows a simple exponential backoff: 1s, 2s, 4s, 8s, ... |\n| [`retry_log_level`](#reactor-req_put-retry_log_level){: #reactor-req_put-retry_log_level } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The log level to emit retry logs at. Can also be set to `false` to disable logging these messages. Defaults to `:warning` |\n| [`retry`](#reactor-req_put-retry){: #reactor-req_put-retry } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | One of `:safe_transient` (default), `:transient`, `fun` or `false` |\n| [`unix_socket`](#reactor-req_put-unix_socket){: #reactor-req_put-unix_socket } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, connect through the given UNIX domain socket |\n| [`url`](#reactor-req_put-url){: #reactor-req_put-url } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request URL |","ref":"dsl-reactor-req.html#options-28"},{"type":"extras","title":"reactor.req_put.wait_for - Reactor.Req","doc":"```elixir\nwait_for names\n```\n\n\nWait for the named step to complete before allowing this one to start.\n\nDesugars to `argument :_, result(step_to_wait_for)`","ref":"dsl-reactor-req.html#reactor-req_put-wait_for"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nwait_for :create_user\n```","ref":"dsl-reactor-req.html#examples-21"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`names`](#reactor-req_put-wait_for-names){: #reactor-req_put-wait_for-names .spark-required} | `atom \\| list(atom)` |  | The name of the step to wait for. |","ref":"dsl-reactor-req.html#arguments-29"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_put-wait_for-description){: #reactor-req_put-wait_for-description } | `String.t` |  | An optional description. |","ref":"dsl-reactor-req.html#options-29"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.WaitFor`","ref":"dsl-reactor-req.html#introspection-28"},{"type":"extras","title":"reactor.req_put.guard - Reactor.Req","doc":"```elixir\nguard fun\n```\n\n\nProvides a flexible method for conditionally executing a step, or replacing it's result.\n\nExpects a two arity function which takes the step's arguments and context and returns one of the following:\n\n- `:cont` - the guard has passed.\n- `{:halt, result}` - the guard has failed - instead of executing the step use the provided result.","ref":"dsl-reactor-req.html#reactor-req_put-guard"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nstep :read_file_via_cache do\n  argument :path, input(:path)\n  run &File.read(&1.path)\n  guard fn %{path: path}, %{cache: cache} ->\n    case Cache.get(cache, path) do\n      {:ok, content} -> {:halt, {:ok, content}}\n      _ -> :cont\n    end\n  end\nend\n\n```","ref":"dsl-reactor-req.html#examples-22"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`fun`](#reactor-req_put-guard-fun){: #reactor-req_put-guard-fun .spark-required} | `(any, any -> any) \\| mfa` |  | The guard function. |","ref":"dsl-reactor-req.html#arguments-30"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_put-guard-description){: #reactor-req_put-guard-description } | `String.t` |  | An optional description of the guard. |","ref":"dsl-reactor-req.html#options-30"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.Guard`","ref":"dsl-reactor-req.html#introspection-29"},{"type":"extras","title":"reactor.req_put.where - Reactor.Req","doc":"```elixir\nwhere predicate\n```\n\n\nOnly execute the surrounding step if the predicate function returns true.\n\nThis is a simple version of `guard` which provides more flexibility at the cost of complexity.","ref":"dsl-reactor-req.html#reactor-req_put-where"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nstep :read_file do\n  argument :path, input(:path)\n  run &File.read(&1.path)\n  where &File.exists?(&1.path)\nend\n\n```","ref":"dsl-reactor-req.html#examples-23"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`predicate`](#reactor-req_put-where-predicate){: #reactor-req_put-where-predicate .spark-required} | `(any -> any) \\| mfa \\| (any, any -> any) \\| mfa` |  | Provide a function which takes the step arguments and optionally the context and returns a boolean value. |","ref":"dsl-reactor-req.html#arguments-31"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_put-where-description){: #reactor-req_put-where-description } | `String.t` |  | An optional description of the guard. |","ref":"dsl-reactor-req.html#options-31"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.Where`","ref":"dsl-reactor-req.html#introspection-30"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Req.Dsl.Put`","ref":"dsl-reactor-req.html#introspection-31"},{"type":"extras","title":"reactor.req_request - Reactor.Req","doc":"```elixir\nreq_request name\n```\n\n\nPerforms a request using `Req.request/2`.\n\nNote that Reactor doesn't validate any options - it simply passes them to the underlying `Req` function and assumes it will validate them.","ref":"dsl-reactor-req.html#reactor-req_request"},{"type":"extras","title":"Nested DSLs - Reactor.Req","doc":"* [wait_for](#reactor-req_request-wait_for)\n * [guard](#reactor-req_request-guard)\n * [where](#reactor-req_request-where)","ref":"dsl-reactor-req.html#nested-dsls-8"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`name`](#reactor-req_request-name){: #reactor-req_request-name .spark-required} | `atom` |  | A unique name for the step. Used when choosing the return value of the Reactor and for arguments into other steps |","ref":"dsl-reactor-req.html#arguments-32"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`adapter`](#reactor-req_request-adapter){: #reactor-req_request-adapter } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Adapter to use to make the actual HTTP request |\n| [`auth`](#reactor-req_request-auth){: #reactor-req_request-auth } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Sets request authentication |\n| [`aws_sigv4`](#reactor-req_request-aws_sigv4){: #reactor-req_request-aws_sigv4 } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, the AWS options to sign request |\n| [`base_url`](#reactor-req_request-base_url){: #reactor-req_request-base_url } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, the request URL is prepended with this base URL |\n| [`body`](#reactor-req_request-body){: #reactor-req_request-body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request body |\n| [`cache_dir`](#reactor-req_request-cache_dir){: #reactor-req_request-cache_dir } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The directory to store the cache |\n| [`cache`](#reactor-req_request-cache){: #reactor-req_request-cache } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If `true`, performs HTTP caching |\n| [`compress_body`](#reactor-req_request-compress_body){: #reactor-req_request-compress_body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, compresses the request body using gzip |\n| [`connect_options`](#reactor-req_request-connect_options){: #reactor-req_request-connect_options } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Dynamically starts (or re-uses already started) Finch pool with the given connection options |\n| [`decode_body`](#reactor-req_request-decode_body){: #reactor-req_request-decode_body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `false`, disables automatic response body decoding |\n| [`decode_json`](#reactor-req_request-decode_json){: #reactor-req_request-decode_json } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Options to pass to `Jason.decode!/2` |\n| [`finch_private`](#reactor-req_request-finch_private){: #reactor-req_request-finch_private } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A map or keyword list of private metadata to add to the Finch request |\n| [`finch_request`](#reactor-req_request-finch_request){: #reactor-req_request-finch_request } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A function that executes the Finch request, defaults to using `Finch.request/3` |\n| [`finch`](#reactor-req_request-finch){: #reactor-req_request-finch } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The Finch pool to use. Defaults to pool automatically started by `Req` |\n| [`form_multipart`](#reactor-req_request-form_multipart){: #reactor-req_request-form_multipart } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as `multipart/form-data` |\n| [`form`](#reactor-req_request-form){: #reactor-req_request-form } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as `application/x-www-form-urlencoded` |\n| [`headers`](#reactor-req_request-headers){: #reactor-req_request-headers } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request headers as a `{key, value}` enumerable (e.g. map, keyword list) |\n| [`http_errors`](#reactor-req_request-http_errors){: #reactor-req_request-http_errors } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | How to manage 4xx and 5xx responses |\n| [`inet6`](#reactor-req_request-inet6){: #reactor-req_request-inet6 } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, uses IPv6 |\n| [`into`](#reactor-req_request-into){: #reactor-req_request-into } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Where to send the response body |\n| [`json`](#reactor-req_request-json){: #reactor-req_request-json } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as JSON |\n| [`max_redirects`](#reactor-req_request-max_redirects){: #reactor-req_request-max_redirects } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The maximum number of redirects, defaults to `10` |\n| [`max_retries`](#reactor-req_request-max_retries){: #reactor-req_request-max_retries } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Maximum number of retry attempts, defaults to `3` (for a total of `4` requests to the server, including the initial one) |\n| [`method`](#reactor-req_request-method){: #reactor-req_request-method } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request method, defaults to `:get` |\n| [`params`](#reactor-req_request-params){: #reactor-req_request-params } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, appends parameters to the request query string (via `put_params` step) |\n| [`path_params_style`](#reactor-req_request-path_params_style){: #reactor-req_request-path_params_style } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | How path params are expressed (via `put_path_params` step) |\n| [`path_params`](#reactor-req_request-path_params){: #reactor-req_request-path_params } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, uses a templated request path (via `put_path_params` step) |\n| [`plug`](#reactor-req_request-plug){: #reactor-req_request-plug } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, calls the given plug instead of making an HTTP request over the network (via `run_plug` step) |\n| [`pool_timeout`](#reactor-req_request-pool_timeout){: #reactor-req_request-pool_timeout } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Pool checkout timeout in milliseconds, defaults to `5000` |\n| [`raw`](#reactor-req_request-raw){: #reactor-req_request-raw } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, disables automatic body decompression (`decompress_body` step) and decoding (`decode_body` step) |\n| [`receive_timeout`](#reactor-req_request-receive_timeout){: #reactor-req_request-receive_timeout } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Socket receive timeout in milliseconds, defaults to `15_000` |\n| [`redirect_trusted`](#reactor-req_request-redirect_trusted){: #reactor-req_request-redirect_trusted } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | By default, authorization credentials are only sent on redirects with the same host, scheme and port. If `:redirect_trusted` is set to `true`, credentials will be sent to any host |\n| [`redirect`](#reactor-req_request-redirect){: #reactor-req_request-redirect } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `false`, disables automatic response redirects |\n| [`request`](#reactor-req_request-request){: #reactor-req_request-request } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A previously built request |\n| [`retry_delay`](#reactor-req_request-retry_delay){: #reactor-req_request-retry_delay } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If not set, which is the default, the retry delay is determined by the value of retry-delay header on HTTP 429/503 responses. If the header is not set, the default delay follows a simple exponential backoff: 1s, 2s, 4s, 8s, ... |\n| [`retry_log_level`](#reactor-req_request-retry_log_level){: #reactor-req_request-retry_log_level } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The log level to emit retry logs at. Can also be set to `false` to disable logging these messages. Defaults to `:warning` |\n| [`retry`](#reactor-req_request-retry){: #reactor-req_request-retry } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | One of `:safe_transient` (default), `:transient`, `fun` or `false` |\n| [`unix_socket`](#reactor-req_request-unix_socket){: #reactor-req_request-unix_socket } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, connect through the given UNIX domain socket |\n| [`url`](#reactor-req_request-url){: #reactor-req_request-url } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request URL |","ref":"dsl-reactor-req.html#options-32"},{"type":"extras","title":"reactor.req_request.wait_for - Reactor.Req","doc":"```elixir\nwait_for names\n```\n\n\nWait for the named step to complete before allowing this one to start.\n\nDesugars to `argument :_, result(step_to_wait_for)`","ref":"dsl-reactor-req.html#reactor-req_request-wait_for"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nwait_for :create_user\n```","ref":"dsl-reactor-req.html#examples-24"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`names`](#reactor-req_request-wait_for-names){: #reactor-req_request-wait_for-names .spark-required} | `atom \\| list(atom)` |  | The name of the step to wait for. |","ref":"dsl-reactor-req.html#arguments-33"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_request-wait_for-description){: #reactor-req_request-wait_for-description } | `String.t` |  | An optional description. |","ref":"dsl-reactor-req.html#options-33"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.WaitFor`","ref":"dsl-reactor-req.html#introspection-32"},{"type":"extras","title":"reactor.req_request.guard - Reactor.Req","doc":"```elixir\nguard fun\n```\n\n\nProvides a flexible method for conditionally executing a step, or replacing it's result.\n\nExpects a two arity function which takes the step's arguments and context and returns one of the following:\n\n- `:cont` - the guard has passed.\n- `{:halt, result}` - the guard has failed - instead of executing the step use the provided result.","ref":"dsl-reactor-req.html#reactor-req_request-guard"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nstep :read_file_via_cache do\n  argument :path, input(:path)\n  run &File.read(&1.path)\n  guard fn %{path: path}, %{cache: cache} ->\n    case Cache.get(cache, path) do\n      {:ok, content} -> {:halt, {:ok, content}}\n      _ -> :cont\n    end\n  end\nend\n\n```","ref":"dsl-reactor-req.html#examples-25"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`fun`](#reactor-req_request-guard-fun){: #reactor-req_request-guard-fun .spark-required} | `(any, any -> any) \\| mfa` |  | The guard function. |","ref":"dsl-reactor-req.html#arguments-34"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_request-guard-description){: #reactor-req_request-guard-description } | `String.t` |  | An optional description of the guard. |","ref":"dsl-reactor-req.html#options-34"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.Guard`","ref":"dsl-reactor-req.html#introspection-33"},{"type":"extras","title":"reactor.req_request.where - Reactor.Req","doc":"```elixir\nwhere predicate\n```\n\n\nOnly execute the surrounding step if the predicate function returns true.\n\nThis is a simple version of `guard` which provides more flexibility at the cost of complexity.","ref":"dsl-reactor-req.html#reactor-req_request-where"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nstep :read_file do\n  argument :path, input(:path)\n  run &File.read(&1.path)\n  where &File.exists?(&1.path)\nend\n\n```","ref":"dsl-reactor-req.html#examples-26"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`predicate`](#reactor-req_request-where-predicate){: #reactor-req_request-where-predicate .spark-required} | `(any -> any) \\| mfa \\| (any, any -> any) \\| mfa` |  | Provide a function which takes the step arguments and optionally the context and returns a boolean value. |","ref":"dsl-reactor-req.html#arguments-35"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_request-where-description){: #reactor-req_request-where-description } | `String.t` |  | An optional description of the guard. |","ref":"dsl-reactor-req.html#options-35"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.Where`","ref":"dsl-reactor-req.html#introspection-34"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Req.Dsl.Request`","ref":"dsl-reactor-req.html#introspection-35"},{"type":"extras","title":"reactor.req_run - Reactor.Req","doc":"```elixir\nreq_run name\n```\n\n\nPerforms a request using `Req.run/2`.\n\nNote that Reactor doesn't validate any options - it simply passes them to the underlying `Req` function and assumes it will validate them.","ref":"dsl-reactor-req.html#reactor-req_run"},{"type":"extras","title":"Nested DSLs - Reactor.Req","doc":"* [wait_for](#reactor-req_run-wait_for)\n * [guard](#reactor-req_run-guard)\n * [where](#reactor-req_run-where)","ref":"dsl-reactor-req.html#nested-dsls-9"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`name`](#reactor-req_run-name){: #reactor-req_run-name .spark-required} | `atom` |  | A unique name for the step. Used when choosing the return value of the Reactor and for arguments into other steps |","ref":"dsl-reactor-req.html#arguments-36"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`adapter`](#reactor-req_run-adapter){: #reactor-req_run-adapter } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Adapter to use to make the actual HTTP request |\n| [`auth`](#reactor-req_run-auth){: #reactor-req_run-auth } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Sets request authentication |\n| [`aws_sigv4`](#reactor-req_run-aws_sigv4){: #reactor-req_run-aws_sigv4 } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, the AWS options to sign request |\n| [`base_url`](#reactor-req_run-base_url){: #reactor-req_run-base_url } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, the request URL is prepended with this base URL |\n| [`body`](#reactor-req_run-body){: #reactor-req_run-body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request body |\n| [`cache_dir`](#reactor-req_run-cache_dir){: #reactor-req_run-cache_dir } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The directory to store the cache |\n| [`cache`](#reactor-req_run-cache){: #reactor-req_run-cache } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If `true`, performs HTTP caching |\n| [`compress_body`](#reactor-req_run-compress_body){: #reactor-req_run-compress_body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, compresses the request body using gzip |\n| [`connect_options`](#reactor-req_run-connect_options){: #reactor-req_run-connect_options } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Dynamically starts (or re-uses already started) Finch pool with the given connection options |\n| [`decode_body`](#reactor-req_run-decode_body){: #reactor-req_run-decode_body } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `false`, disables automatic response body decoding |\n| [`decode_json`](#reactor-req_run-decode_json){: #reactor-req_run-decode_json } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Options to pass to `Jason.decode!/2` |\n| [`finch_private`](#reactor-req_run-finch_private){: #reactor-req_run-finch_private } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A map or keyword list of private metadata to add to the Finch request |\n| [`finch_request`](#reactor-req_run-finch_request){: #reactor-req_run-finch_request } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A function that executes the Finch request, defaults to using `Finch.request/3` |\n| [`finch`](#reactor-req_run-finch){: #reactor-req_run-finch } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The Finch pool to use. Defaults to pool automatically started by `Req` |\n| [`form_multipart`](#reactor-req_run-form_multipart){: #reactor-req_run-form_multipart } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as `multipart/form-data` |\n| [`form`](#reactor-req_run-form){: #reactor-req_run-form } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as `application/x-www-form-urlencoded` |\n| [`headers`](#reactor-req_run-headers){: #reactor-req_run-headers } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request headers as a `{key, value}` enumerable (e.g. map, keyword list) |\n| [`http_errors`](#reactor-req_run-http_errors){: #reactor-req_run-http_errors } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | How to manage 4xx and 5xx responses |\n| [`inet6`](#reactor-req_run-inet6){: #reactor-req_run-inet6 } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, uses IPv6 |\n| [`into`](#reactor-req_run-into){: #reactor-req_run-into } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Where to send the response body |\n| [`json`](#reactor-req_run-json){: #reactor-req_run-json } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, encodes the request body as JSON |\n| [`max_redirects`](#reactor-req_run-max_redirects){: #reactor-req_run-max_redirects } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The maximum number of redirects, defaults to `10` |\n| [`max_retries`](#reactor-req_run-max_retries){: #reactor-req_run-max_retries } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Maximum number of retry attempts, defaults to `3` (for a total of `4` requests to the server, including the initial one) |\n| [`method`](#reactor-req_run-method){: #reactor-req_run-method } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request method, defaults to `:get` |\n| [`params`](#reactor-req_run-params){: #reactor-req_run-params } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, appends parameters to the request query string (via `put_params` step) |\n| [`path_params_style`](#reactor-req_run-path_params_style){: #reactor-req_run-path_params_style } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | How path params are expressed (via `put_path_params` step) |\n| [`path_params`](#reactor-req_run-path_params){: #reactor-req_run-path_params } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, uses a templated request path (via `put_path_params` step) |\n| [`plug`](#reactor-req_run-plug){: #reactor-req_run-plug } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, calls the given plug instead of making an HTTP request over the network (via `run_plug` step) |\n| [`pool_timeout`](#reactor-req_run-pool_timeout){: #reactor-req_run-pool_timeout } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Pool checkout timeout in milliseconds, defaults to `5000` |\n| [`raw`](#reactor-req_run-raw){: #reactor-req_run-raw } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `true`, disables automatic body decompression (`decompress_body` step) and decoding (`decode_body` step) |\n| [`receive_timeout`](#reactor-req_run-receive_timeout){: #reactor-req_run-receive_timeout } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | Socket receive timeout in milliseconds, defaults to `15_000` |\n| [`redirect_trusted`](#reactor-req_run-redirect_trusted){: #reactor-req_run-redirect_trusted } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | By default, authorization credentials are only sent on redirects with the same host, scheme and port. If `:redirect_trusted` is set to `true`, credentials will be sent to any host |\n| [`redirect`](#reactor-req_run-redirect){: #reactor-req_run-redirect } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set to `false`, disables automatic response redirects |\n| [`request`](#reactor-req_run-request){: #reactor-req_run-request } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | A previously built request |\n| [`retry_delay`](#reactor-req_run-retry_delay){: #reactor-req_run-retry_delay } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If not set, which is the default, the retry delay is determined by the value of retry-delay header on HTTP 429/503 responses. If the header is not set, the default delay follows a simple exponential backoff: 1s, 2s, 4s, 8s, ... |\n| [`retry_log_level`](#reactor-req_run-retry_log_level){: #reactor-req_run-retry_log_level } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The log level to emit retry logs at. Can also be set to `false` to disable logging these messages. Defaults to `:warning` |\n| [`retry`](#reactor-req_run-retry){: #reactor-req_run-retry } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | One of `:safe_transient` (default), `:transient`, `fun` or `false` |\n| [`unix_socket`](#reactor-req_run-unix_socket){: #reactor-req_run-unix_socket } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | If set, connect through the given UNIX domain socket |\n| [`url`](#reactor-req_run-url){: #reactor-req_run-url } | `nil \\| Reactor.Template.Element \\| Reactor.Template.Input \\| Reactor.Template.Result \\| Reactor.Template.Value` |  | The request URL |","ref":"dsl-reactor-req.html#options-36"},{"type":"extras","title":"reactor.req_run.wait_for - Reactor.Req","doc":"```elixir\nwait_for names\n```\n\n\nWait for the named step to complete before allowing this one to start.\n\nDesugars to `argument :_, result(step_to_wait_for)`","ref":"dsl-reactor-req.html#reactor-req_run-wait_for"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nwait_for :create_user\n```","ref":"dsl-reactor-req.html#examples-27"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`names`](#reactor-req_run-wait_for-names){: #reactor-req_run-wait_for-names .spark-required} | `atom \\| list(atom)` |  | The name of the step to wait for. |","ref":"dsl-reactor-req.html#arguments-37"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_run-wait_for-description){: #reactor-req_run-wait_for-description } | `String.t` |  | An optional description. |","ref":"dsl-reactor-req.html#options-37"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.WaitFor`","ref":"dsl-reactor-req.html#introspection-36"},{"type":"extras","title":"reactor.req_run.guard - Reactor.Req","doc":"```elixir\nguard fun\n```\n\n\nProvides a flexible method for conditionally executing a step, or replacing it's result.\n\nExpects a two arity function which takes the step's arguments and context and returns one of the following:\n\n- `:cont` - the guard has passed.\n- `{:halt, result}` - the guard has failed - instead of executing the step use the provided result.","ref":"dsl-reactor-req.html#reactor-req_run-guard"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nstep :read_file_via_cache do\n  argument :path, input(:path)\n  run &File.read(&1.path)\n  guard fn %{path: path}, %{cache: cache} ->\n    case Cache.get(cache, path) do\n      {:ok, content} -> {:halt, {:ok, content}}\n      _ -> :cont\n    end\n  end\nend\n\n```","ref":"dsl-reactor-req.html#examples-28"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`fun`](#reactor-req_run-guard-fun){: #reactor-req_run-guard-fun .spark-required} | `(any, any -> any) \\| mfa` |  | The guard function. |","ref":"dsl-reactor-req.html#arguments-38"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_run-guard-description){: #reactor-req_run-guard-description } | `String.t` |  | An optional description of the guard. |","ref":"dsl-reactor-req.html#options-38"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.Guard`","ref":"dsl-reactor-req.html#introspection-37"},{"type":"extras","title":"reactor.req_run.where - Reactor.Req","doc":"```elixir\nwhere predicate\n```\n\n\nOnly execute the surrounding step if the predicate function returns true.\n\nThis is a simple version of `guard` which provides more flexibility at the cost of complexity.","ref":"dsl-reactor-req.html#reactor-req_run-where"},{"type":"extras","title":"Examples - Reactor.Req","doc":"```\nstep :read_file do\n  argument :path, input(:path)\n  run &File.read(&1.path)\n  where &File.exists?(&1.path)\nend\n\n```","ref":"dsl-reactor-req.html#examples-29"},{"type":"extras","title":"Arguments - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`predicate`](#reactor-req_run-where-predicate){: #reactor-req_run-where-predicate .spark-required} | `(any -> any) \\| mfa \\| (any, any -> any) \\| mfa` |  | Provide a function which takes the step arguments and optionally the context and returns a boolean value. |","ref":"dsl-reactor-req.html#arguments-39"},{"type":"extras","title":"Options - Reactor.Req","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`description`](#reactor-req_run-where-description){: #reactor-req_run-where-description } | `String.t` |  | An optional description of the guard. |","ref":"dsl-reactor-req.html#options-39"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Dsl.Where`","ref":"dsl-reactor-req.html#introspection-38"},{"type":"extras","title":"Introspection - Reactor.Req","doc":"Target: `Reactor.Req.Dsl.Run`\n\n\n\n .spark-required::after { content: \"*\"; color: red !important; }","ref":"dsl-reactor-req.html#introspection-39"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}